@@ -32,7 +32,6 @@
 #include <openvdb/util/logging.h>
 #include <math.h>
 
-
 ////////////////////////////////////////
 
 // BufferObject
@@ -78,6 +77,10 @@ BufferObject::render() const
 
     glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, mIndexBuffer);
     glDrawElements(mPrimType, mPrimNum, GL_UNSIGNED_INT, 0);
+    
+    // disable client-side capabilities
+    if (usesColorBuffer) glDisableClientState(GL_COLOR_ARRAY);
+    if (usesNormalBuffer) glDisableClientState(GL_NORMAL_ARRAY);
 
     // release vbo's
     glBindBuffer(GL_ARRAY_BUFFER, 0);
@@ -313,6 +316,8 @@ ViewportModule::ViewportModule():
 void
 ViewportModule::render()
 {
+    if (!mIsVisible) return;
+
     /// @todo use VBO's
 
     // Ground plane
@@ -369,7 +374,7 @@ ViewportModule::render()
 
     glScalef(mAxisGnomonScale, mAxisGnomonScale, mAxisGnomonScale);
 
-    glLineWidth(2.0);
+    glLineWidth(1.0);
 
     glBegin(GL_LINES);
     glColor3f(1.0f, 0.0f, 0.0f);
@@ -398,16 +403,17 @@ ViewportModule::render()
 
 // Tree topology render module
 
-TreeTopologyModule::TreeTopologyModule(const openvdb::GridCPtrVec& grids):
-    mGrids(grids),
-    mBufferObjects(grids.size()),
+TreeTopologyModule::TreeTopologyModule(const openvdb::GridBase::ConstPtr& grid):
+    mGrid(grid),
     mIsInitialized(false)
 {
     mShader.setVertShader(
         "#version 120\n"
         "void main() {\n"
         "gl_FrontColor = gl_Color;\n"
-        "gl_Position =  ftransform();}\n");
+        "gl_Position =  ftransform();\n"
+        "gl_ClipVertex = gl_ModelViewMatrix * gl_Vertex;\n"
+        "}\n");
 
     mShader.setFragShader(
         "#version 120\n"
@@ -424,16 +430,11 @@ TreeTopologyModule::init()
     mIsInitialized = true;
 
     // extract grid topology
-    for (size_t n = 0, N = mGrids.size(); n < N; ++n) {
+    TreeTopologyOp drawTopology(mBufferObject);
 
-        openvdb::GridBase::ConstPtr grid = mGrids[n];
-
-        TreeTopologyOp drawTopology(mBufferObjects[n]);
-
-        if (!util::processTypedGrid(grid, drawTopology)) {
-            OPENVDB_LOG_INFO("Ignoring unrecognized grid type"
-                " during tree topology module initialization.");
-        }
+    if (!util::processTypedGrid(mGrid, drawTopology)) {
+        OPENVDB_LOG_INFO("Ignoring unrecognized grid type"
+            " during tree topology module initialization.");
     }
 }
 
@@ -441,25 +442,106 @@ TreeTopologyModule::init()
 void
 TreeTopologyModule::render()
 {
+    if (!mIsVisible) return;
     if (!mIsInitialized) init();
 
     mShader.startShading();
 
-    for (size_t n = 0, N = mBufferObjects.size(); n < N; ++n) {
-       mBufferObjects[n].render();
-    }
+    mBufferObject.render();
 
     mShader.stopShading();
 }
 
+////////////////////////////////////////
+
+// Active value render module
+
+ActiveValueModule::ActiveValueModule(const openvdb::GridBase::ConstPtr& grid):
+    mGrid(grid),
+    mIsInitialized(false)
+{
+    mFlatShader.setVertShader(
+        "#version 120\n"
+        "void main() {\n"
+        "gl_FrontColor = gl_Color;\n"
+        "gl_Position =  ftransform();\n"
+        "gl_ClipVertex = gl_ModelViewMatrix * gl_Vertex;\n"
+        "}\n");
+
+    mFlatShader.setFragShader(
+        "#version 120\n"
+         "void main() {\n"
+            "gl_FragColor = gl_Color;}\n");
+
+    mFlatShader.build();
+
+    mSurfaceShader.setVertShader(
+        "#version 120\n"
+        "varying vec3 normal;\n"
+        "void main() {\n"
+            "gl_FrontColor = gl_Color;\n"
+            "normal = normalize(gl_NormalMatrix * gl_Normal);\n"
+            "gl_Position =  ftransform();\n"
+            "gl_ClipVertex = gl_ModelViewMatrix * gl_Vertex;\n"
+        "}\n");
+
+
+    mSurfaceShader.setFragShader(
+        "#version 120\n"
+        "varying vec3 normal;\n"
+        "void main() {\n"
+            "vec3 normalized_normal = normalize(normal);\n"
+            "float w = 0.5 * (1.0 + dot(normalized_normal, vec3(0.0, 1.0, 0.0)));\n"
+            "vec4 diffuseColor = w * gl_Color + (1.0 - w) * (gl_Color * 0.3);\n"
+            "gl_FragColor = diffuseColor;\n"
+        "}\n");
+
+    mSurfaceShader.build();
+}
+
+
+void
+ActiveValueModule::init()
+{
+    mIsInitialized = true;
+
+    ActiveScalarValuesOp drawScalars(mInteriorBuffer, mSurfaceBuffer);
+
+    if (!util::processTypedScalarGrid(mGrid, drawScalars)) {
+
+        ActiveVectorValuesOp drawVectors(mVectorBuffer);
+
+        if(!util::processTypedVectorGrid(mGrid, drawVectors)) {
+            OPENVDB_LOG_INFO("Ignoring unrecognized grid type"
+                " during active value module initialization.");
+        }
+    }
+}
+
+
+void
+ActiveValueModule::render()
+{
+    if (!mIsVisible) return;
+    if (!mIsInitialized) init();
+
+    mFlatShader.startShading();
+        mInteriorBuffer.render();
+        mVectorBuffer.render();
+    mFlatShader.stopShading();
+
+    mSurfaceShader.startShading();
+        mSurfaceBuffer.render();
+    mSurfaceShader.stopShading();
+}
+
 
 ////////////////////////////////////////
 
 // Meshing module
 
-MeshModule::MeshModule(const openvdb::GridCPtrVec& grids):
-    mGrids(grids),
-    mBufferObjects(grids.size()),
+MeshModule::MeshModule(const openvdb::GridBase::ConstPtr& grid):
+    mGrid(grid),
     mIsInitialized(false)
 {
     mShader.setVertShader(
@@ -468,6 +550,7 @@ MeshModule::MeshModule(const openvdb::GridCPtrVec& grids):
         "void main() {\n"
             "normal = normalize(gl_NormalMatrix * gl_Normal);\n"
             "gl_Position =  ftransform();\n"
+            "gl_ClipVertex = gl_ModelViewMatrix * gl_Vertex;\n"
         "}\n");
 
     mShader.setFragShader(
@@ -491,17 +574,11 @@ MeshModule::init()
 {
     mIsInitialized = true;
 
-    // extract grid topology
-    for (size_t n = 0, N = mGrids.size(); n < N; ++n) {
+    MeshOp drawMesh(mBufferObject);
 
-        openvdb::GridBase::ConstPtr grid = mGrids[n];
-
-        MeshOp drawMesh(mBufferObjects[n]);
-
-        if (!util::processTypedScalarGrid(grid, drawMesh)) {
-            OPENVDB_LOG_INFO(
-                "Ignoring non-scalar grid type during mesh module initialization.");
-        }
+    if (!util::processTypedScalarGrid(mGrid, drawMesh)) {
+        OPENVDB_LOG_INFO(
+            "Ignoring non-scalar grid type during mesh module initialization.");
     }
 }
 
@@ -509,13 +586,12 @@ MeshModule::init()
 void
 MeshModule::render()
 {
+    if (!mIsVisible) return;
     if (!mIsInitialized) init();
 
     mShader.startShading();
 
-    for (size_t n = 0, N = mBufferObjects.size(); n < N; ++n) {
-        mBufferObjects[n].render();
-    }
+    mBufferObject.render();
 
     mShader.stopShading();
 }
