@@ -35,6 +35,8 @@
 #include "Math.h"
 #include "Vec3.h"
 
+namespace tbb { class split; } // forward declaration
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -258,24 +260,30 @@ private:
 class CoordBBox
 {
 public:
+    typedef uint64_t         Index64;
     typedef Coord::ValueType ValueType;
-
-    // Empty constructor produces an invalid (i.e. empty) bbox!
+    
+    /// @brief The default constructor produces an empty bounding box.
     CoordBBox(): mMin(Coord::max()), mMax(Coord::min()) {}
-    // Construct bbox from min and max
+    /// @brief Construct a bounding box with the given @a min and @a max bounds.
     CoordBBox(const Coord& min, const Coord& max): mMin(min), mMax(max) {}
+    /// @brief Splitting constructor for use in TBB ranges
+    /// @note The other bounding box is assumed to be divisible.
+    CoordBBox(CoordBBox& other, const tbb::split&): mMin(other.mMin), mMax(other.mMax)
+    {
+        assert(this->is_divisible());
+        const size_t n = this->maxExtent();
+        mMax[n] = (mMin[n] + mMax[n]) >> 1;
+        other.mMin[n] = mMax[n] + 1;
+    }
 
     static CoordBBox createCube(const Coord& min, ValueType dim)
     {
         return CoordBBox(min, min.offsetBy(dim - 1));
     }
 
-    /// @return an infinite bounding box defined from the value range of Coord
-    static const CoordBBox& inf()
-    {
-        static const CoordBBox sInf(Coord::min(), Coord::max());
-        return sInf;
-    }
+    /// Return an "infinite" bounding box, as defined by the Coord value range.
+    static CoordBBox inf() { return CoordBBox(Coord::min(), Coord::max()); }
 
     const Coord& min() const { return mMin; }
     const Coord& max() const { return mMax; }
@@ -295,23 +303,30 @@ public:
     bool operator!=(const CoordBBox& rhs) const { return !(*this == rhs); }
 
     bool empty() const { return (mMin[0] > mMax[0] || mMin[1] > mMax[1] || mMin[2] > mMax[2]); }
+    //@{
+    /// Return @c true if this bounding box is nonempty
     operator bool() const { return !this->empty(); }
-    /// Return @c true if this bbox is not empty
     bool hasVolume() const { return !this->empty(); }
-    /// @brief Return the integer volume of coordinates spanned by this bbox.
-    /// @note Since coordinates are inclusive, a bounding box with min = max
-    /// has volume one.
-    ValueType volume() const { const Coord d = this->dim(); return d[0] * d[1] * d[2]; }
+    //@}
 
-    /// Return the floating-point position of the center of this bbox
+    /// Return the floating-point position of the center of this bounding box.
     Vec3d getCenter() const { return 0.5 * Vec3d((mMin + mMax).asPointer()); }
 
-    /// @brief Return the dimensions of the coordinates spanned by this bbox.
+    /// @brief Return the dimensions of the coordinates spanned by this bounding box.
     /// @note Since coordinates are inclusive, a bounding box with min = max
     /// has dimensions of (1, 1, 1).
     Coord dim() const { return mMax.offsetBy(1) - mMin; }
     /// @todo deprecate - use dim instead
     Coord extents() const { return this->dim(); }
+    /// @brief Return the integer volume of coordinates spanned by this bounding box.
+    /// @note Since coordinates are inclusive, a bounding box with min = max has volume one.
+    Index64 volume() const
+    {
+        const Coord d = this->dim();
+        return Index64(d[0]) * Index64(d[1]) * Index64(d[2]);
+    }
+    /// Return @c true if this bounding box can be subdivided [mainly for use by TBB].
+    bool is_divisible() const { return mMin[0]<mMax[0] && mMin[1]<mMax[1] && mMin[2]<mMax[2]; }
 
     /// Return the index (0, 1 or 2) of the longest axis.
     size_t maxExtent() const
@@ -353,16 +368,23 @@ public:
     /// Union this bounding box with the given bounding box.
     void expand(const CoordBBox& bbox)
     {
-        mMin.minComponent(bbox.min());
-        mMax.maxComponent(bbox.max());
+          mMin.minComponent(bbox.min());
+          mMax.maxComponent(bbox.max());
     }
-    // Union this bbox with the cubical bbox defined from min and dim
+    /// Intersect this bounding box with the given bounding box.
+    void intersect(const CoordBBox& bbox)
+    {
+        mMin.maxComponent(bbox.min());
+        mMax.minComponent(bbox.max());
+    }
+    /// @brief Union this bounding box with the cubical bounding box
+    /// of the given size and with the given minimum coordinates.
     void expand(const Coord& min, Coord::ValueType dim)
     {
         mMin.minComponent(min);
         mMax.maxComponent(min.offsetBy(dim-1));
     }
-    /// Translate this bounding box by \f$(t_x, t_y, t_z)\f$.
+    /// Translate this bounding box by @f$(t_x, t_y, t_z)@f$.
     void translate(const Coord& t) { mMin += t; mMax += t; }
 
     /// Unserialize this bounding box from the given stream.
