@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -29,7 +29,7 @@
 ///////////////////////////////////////////////////////////////////////////
 
 #include "Transform.h"
-#include "Frustum.h" // for LegacyFrustum
+#include "LegacyFrustum.h"
 
 #include <openvdb/version.h>
 #include <sstream>
@@ -79,16 +79,6 @@ Transform::createLinearTransform(const Mat4R& m)
     return Transform::Ptr(new Transform(MapBase::Ptr(new AffineMap(m))));
 }
 
-Transform::Ptr
-Transform::createFrustumTransform(const CoordBBox& bbox, double taper,
-    double depth, double voxelDim)
-{
-    const BBoxd bb(bbox.min().asVec3d(), bbox.max().asVec3d()); 
-    return Transform::Ptr(new  Transform(
-        NonlinearFrustumMap(bb, taper, depth).preScale(Vec3d(voxelDim, voxelDim, voxelDim))));
-}
-
-
 Transform::Ptr
 Transform::createFrustumTransform(const BBoxd& bbox, double taper,
     double depth, double voxelDim)
@@ -109,8 +99,8 @@ Transform::read(std::istream& is)
 
     if (io::getFormatVersion(is) < OPENVDB_FILE_VERSION_NEW_TRANSFORM) {
         // Handle old-style transforms.
-        
-        if(type == "LinearTransform") { 
+
+        if(type == "LinearTransform") {
             // First read in the old transform's base class.
             Coord tmpMin, tmpMax;
             is.read(reinterpret_cast<char*>(&tmpMin), sizeof(Coord::ValueType) * 3);
@@ -129,14 +119,14 @@ Transform::read(std::istream& is)
             mMap = simplify(affineMap);
 
         } else if (type == "FrustumTransform") {
-          
-            LegacyFrustum legacyFrustum(is);
-                  
+
+            internal::LegacyFrustum legacyFrustum(is);
+
             CoordBBox bb      = legacyFrustum.getBBox();
             BBoxd   bbox(bb.min().asVec3d(), bb.max().asVec3d());
             double  taper     = legacyFrustum.getTaper();
             double  depth     = legacyFrustum.getDepth();
-            
+
             double nearPlaneWidth = legacyFrustum.getNearPlaneWidth();
             double nearPlaneDist  = legacyFrustum.getNearPlaneDist();
             Mat4d camxform        = legacyFrustum.getCamXForm();
@@ -145,15 +135,16 @@ Transform::read(std::istream& is)
             Mat4d xform(Mat4d::identity());
             xform.setToTranslation(Vec3d(0,0, -nearPlaneDist));
             xform.preScale(Vec3d(nearPlaneWidth, nearPlaneWidth, -nearPlaneWidth));
-            
+
             MapBase::Ptr linearMap(simplify(AffineMap(xform * camxform).getAffineMap()));
 
-            // note that the depth is scaled on the nearPlaneSize. 
+            // note that the depth is scaled on the nearPlaneSize.
             // the linearMap will uniformly scale the frustum to the correct size
             // and rotate to align with the camera
-            mMap = MapBase::Ptr(new NonlinearFrustumMap(bbox, taper, depth/nearPlaneWidth, linearMap));
+            mMap = MapBase::Ptr(new NonlinearFrustumMap(
+                bbox, taper, depth/nearPlaneWidth, linearMap));
+
 
-            
         } else {
             OPENVDB_THROW(IoError, "Transforms of type " + type + " are no longer supported");
         }
@@ -184,6 +175,20 @@ Transform::write(std::ostream& os) const
 
 ////////////////////////////////////////
 
+bool
+Transform::isIdentity() const 
+{
+    if (mMap->isLinear()) {
+        return mMap->getAffineMap()->isIdentity();
+    } else if ( mMap->isType<NonlinearFrustumMap>() ) {
+        NonlinearFrustumMap::Ptr frustum 
+            = boost::static_pointer_cast<NonlinearFrustumMap, MapBase>(mMap);
+        return frustum->isIdentity();
+    }
+    // unknown nonlinear map type
+    return false;
+}
+
 void
 Transform::preRotate(double radians, const Axis axis)
 {
@@ -215,6 +220,44 @@ Transform::preShear(double shear, Axis axis0, Axis axis1)
     mMap = mMap->preShear(shear, axis0, axis1);
 }
 
+void
+Transform::preMult(const Mat4d& m)
+{   
+    if (mMap->isLinear()) {
+
+        const Mat4d currentMat4 = mMap->getAffineMap()->getMat4();
+        const Mat4d newMat4 = m * currentMat4;
+                
+        AffineMap::Ptr affineMap( new AffineMap( newMat4) );
+        mMap = simplify(affineMap);
+
+    } else if (mMap->isType<NonlinearFrustumMap>() ) {
+        
+        NonlinearFrustumMap::Ptr currentFrustum =  
+            boost::static_pointer_cast<NonlinearFrustumMap, MapBase>(mMap);
+        
+        const Mat4d currentMat4 = currentFrustum->secondMap().getMat4();
+        const Mat4d newMat4 = m * currentMat4;
+
+        AffineMap affine(newMat4);
+        
+        NonlinearFrustumMap::Ptr frustum( new NonlinearFrustumMap( currentFrustum->getBBox(),
+                                                                   currentFrustum->getTaper(),
+                                                                   currentFrustum->getDepth(),
+                                                                   affine.copy() ) );
+        mMap = boost::static_pointer_cast<MapBase, NonlinearFrustumMap>( frustum );
+    }
+        
+}    
+
+void
+Transform::preMult(const Mat3d& m)
+{
+    Mat4d mat4 = Mat4d::identity();
+    mat4.setMat3(m);
+    preMult(mat4);
+}
+
 void
 Transform::postRotate(double radians, const Axis axis)
 {
@@ -247,6 +290,45 @@ Transform::postShear(double shear, Axis axis0, Axis axis1)
 }
 
 
+void
+Transform::postMult(const Mat4d& m)
+{   
+    if (mMap->isLinear()) {
+
+        const Mat4d currentMat4 = mMap->getAffineMap()->getMat4();
+        const Mat4d newMat4 = currentMat4 * m;
+                
+        AffineMap::Ptr affineMap( new AffineMap( newMat4) );
+        mMap = simplify(affineMap);
+
+    } else if (mMap->isType<NonlinearFrustumMap>() ) {
+        
+        NonlinearFrustumMap::Ptr currentFrustum =  
+            boost::static_pointer_cast<NonlinearFrustumMap, MapBase>(mMap);
+        
+        const Mat4d currentMat4 = currentFrustum->secondMap().getMat4();
+        const Mat4d newMat4 =  currentMat4 * m;
+
+        AffineMap affine(newMat4);
+        
+        NonlinearFrustumMap::Ptr frustum( new NonlinearFrustumMap( currentFrustum->getBBox(),
+                                                                   currentFrustum->getTaper(),
+                                                                   currentFrustum->getDepth(),
+                                                                   affine.copy() ) );
+        mMap = boost::static_pointer_cast<MapBase, NonlinearFrustumMap>( frustum );
+    }
+        
+}    
+
+void
+Transform::postMult(const Mat3d& m)
+{
+    Mat4d mat4 = Mat4d::identity();
+    mat4.setMat3(m);
+    postMult(mat4);
+}
+
+
 ////////////////////////////////////////
 
 // Utility methods
@@ -289,17 +371,17 @@ bool
 Transform::operator==(const Transform& other) const
 {
     if (!this->voxelSize().eq(other.voxelSize())) return false;
-   
+
     if (this->mapType() == other.mapType()) {
         return this->baseMap()->isEqual(*other.baseMap());
     }
 
     if (this->isLinear() && other.isLinear()) {
         // promote both maps to mat4 form and compare
-        return  ( *(this->baseMap()->getAffineMap()) == 
+        return  ( *(this->baseMap()->getAffineMap()) ==
                   *(other.baseMap()->getAffineMap()) );
     }
-    
+
     return this->baseMap()->isEqual(*other.baseMap());
 }
 
@@ -399,47 +481,10 @@ operator<<(std::ostream& os, const Transform& t)
 }
 
 
-    //// --- Deprecated
-// Accum transformations
-
-
-OPENVDB_DEPRECATED void
-Transform::rotate(double radians, const Axis axis)
-{
-   mMap = mMap->preRotate(radians, axis);
-}
-
-OPENVDB_DEPRECATED void
-Transform::translate(const Vec3d& t)
-{
-    mMap = mMap->postTranslate(t);
-}
-
-OPENVDB_DEPRECATED void
-Transform::scale(const Vec3d& s)
-{
-    mMap = mMap->preScale(s);
-}
-
-OPENVDB_DEPRECATED void
-Transform::scale(double s)
-{
-    const Vec3d vec(s,s,s);
-    mMap = mMap->preScale(vec);
-}
-
-OPENVDB_DEPRECATED void
-Transform::shear(double shear, Axis axis0, Axis axis1)
-{
-    mMap = mMap->preShear(shear, axis0, axis1);
-}
-
-
-
 } // namespace math
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
