@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -40,11 +40,11 @@
 #include <tbb/blocked_range.h>
 #include <tbb/parallel_reduce.h>
 #include <openvdb/Grid.h>
-#include <openvdb/Platform.h> // for round()
 #include <openvdb/Types.h>
 #include <openvdb/math/Math.h> // for isApproxEqual()
 #include <openvdb/util/NullInterrupter.h>
 #include "Interpolation.h"
+#include "LevelSetRebuild.h" // for doLevelSetRebuild()
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -201,13 +201,6 @@ public:
     /// @see util/NullInterrupter.h for interrupter interface requirements.
     template<typename InterrupterType> void setInterrupter(InterrupterType&);
 
-    /// @brief Allow processing to be aborted by providing a callable that takes
-    /// no arguments and that returns @c true when processing should be aborted
-    /// and @c false if it should continue.
-    /// @details The supplied callable will be called periodically during processing.
-    /// @deprecated Use setInterrupter() instead.
-    OPENVDB_DEPRECATED void setInterrupt(const InterruptFunc& f) { mInterrupt = f; }
-
     template<typename Sampler, typename GridT, typename Transformer>
     void transformGrid(const Transformer&,
         const GridT& inGrid, GridT& outGrid) const;
@@ -389,14 +382,20 @@ private:
 };
 
 
+/// The normal entry points for resampling are the resampleToMatch() functions,
+/// which correctly handle level set grids under scaling and shearing.
+/// doResampleToMatch() is mainly for internal use but is typically faster
+/// for level sets, and correct provided that no scaling or shearing is needed.
+///
+/// @warning Do not use this function to scale or shear a level set grid.
 template<typename Sampler, typename Interrupter, typename GridType>
 inline void
-resampleToMatch(const GridType& inGrid, GridType& outGrid, Interrupter& interrupter)
+doResampleToMatch(const GridType& inGrid, GridType& outGrid, Interrupter& interrupter)
 {
     ABTransform xform(inGrid.transform(), outGrid.transform());
 
     if (Sampler::consistent() && xform.isIdentity()) {
-        // If the transform of the input and output are identical, the
+        // If the transforms of the input and output are identical, the
         // output tree is simply a deep copy of the input tree.
         outGrid.setTree(inGrid.tree().copy());
     } else if (xform.isAffine()) {
@@ -422,6 +421,49 @@ resampleToMatch(const GridType& inGrid, GridType& outGrid, Interrupter& interrup
 }
 
 
+template<typename Sampler, typename Interrupter, typename GridType>
+inline void
+resampleToMatch(const GridType& inGrid, GridType& outGrid, Interrupter& interrupter)
+{
+    if (inGrid.getGridClass() == GRID_LEVEL_SET) {
+        // If the input grid is a level set, resample it using the level set rebuild tool.
+
+        if (inGrid.constTransform() == outGrid.constTransform()) {
+            // If the transforms of the input and output grids are identical,
+            // the output tree is simply a deep copy of the input tree.
+            outGrid.setTree(inGrid.tree().copy());
+            return;
+        }
+
+        // If the output grid is a level set, resample the input grid to have the output grid's
+        // background value.  Otherwise, preserve the input grid's background value.
+        typedef typename GridType::ValueType ValueT;
+        const ValueT halfWidth = ((outGrid.getGridClass() == openvdb::GRID_LEVEL_SET)
+            ? ValueT(outGrid.background() * (1.0 / outGrid.voxelSize()[0]))
+            : ValueT(inGrid.background() * (1.0 / inGrid.voxelSize()[0])));
+
+        typename GridType::Ptr tempGrid;
+        try {
+            tempGrid = doLevelSetRebuild(inGrid, /*iso=*/zeroVal<ValueT>(),
+                /*exWidth=*/halfWidth, /*inWidth=*/halfWidth,
+                &outGrid.constTransform(), &interrupter);
+        } catch (TypeError&) {
+            // The input grid is classified as a level set, but it has a value type
+            // that is not supported by the level set rebuild tool.  Fall back to
+            // using the generic resampler.
+            tempGrid.reset();
+        }
+        if (tempGrid) {
+            outGrid.setTree(tempGrid->treePtr());
+            return;
+        }
+    }
+
+    // If the input grid is not a level set, use the generic resampler.
+    doResampleToMatch<Sampler>(inGrid, outGrid, interrupter);
+}
+
+
 template<typename Sampler, typename GridType>
 inline void
 resampleToMatch(const GridType& inGrid, GridType& outGrid)
@@ -674,7 +716,7 @@ public:
 
     RangeProcessor(const Transformer& xform, const CoordBBox& b, const TreeT& inTree):
         mIsRoot(false), mXform(xform), mBBox(b),
-        mInTree(inTree), mOutTree(new TreeT(inTree.getBackground())),
+        mInTree(inTree), mOutTree(new TreeT(inTree.background())),
         mInAcc(mInTree), mOutAcc(*mOutTree)
     {}
 
@@ -686,7 +728,7 @@ public:
         mXform(other.mXform),
         mBBox(other.mBBox),
         mInTree(other.mInTree),
-        mOutTree(new TreeT(mInTree.getBackground())),
+        mOutTree(new TreeT(mInTree.background())),
         mInAcc(mInTree),
         mOutAcc(*mOutTree),
         mInterrupt(other.mInterrupt)
@@ -722,7 +764,7 @@ public:
             TileIterT i = r.iterator();
             // Skip voxels and background tiles.
             if (!i.isTileValue()) continue;
-            if (!i.isValueOn() && math::isApproxEqual(*i, mOutTree->getBackground())) continue;
+            if (!i.isValueOn() && math::isApproxEqual(*i, mOutTree->background())) continue;
 
             CoordBBox bbox;
             i.getBoundingBox(bbox);
@@ -939,6 +981,6 @@ GridResampler::transformBBox(
 
 #endif // OPENVDB_TOOLS_GRIDTRANSFORMER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
