@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -28,6 +28,8 @@
 //
 ///////////////////////////////////////////////////////////////////////////
 //
+///////////////////////////////////////////////////////////////////////////
+//
 /// @author Ken Museth
 ///
 /// @file LevelSetAdvect.h
@@ -38,7 +40,7 @@
 #define OPENVDB_TOOLS_LEVEL_SET_ADVECT_HAS_BEEN_INCLUDED
 
 #include "LevelSetTracker.h"
-#include <openvdb/tools/GridTransformer.h>// for BoxSampler etc
+#include "Interpolation.h" // for BoxSampler, etc.
 #include <openvdb/math/FiniteDifference.h>
 
 namespace openvdb {
@@ -58,7 +60,7 @@ namespace tools {
 /// this implies the input grid must contain velocities in world
 /// coordinates.
 
-/// @brief Sime wrapper class for a velocity grid
+/// @brief Thin wrapper class for a velocity grid
 /// @note Consider replacing BoxSampler with StaggeredBoxSampler
 template <typename VelGridT, typename Interpolator = BoxSampler>
 class DiscreteField
@@ -218,12 +220,15 @@ public:
     /// @note A grainsize of 0 or less disables multi-threading!
     void setGrainSize(int grainsize) { mTracker.setGrainSize(grainsize); }
 
-    /// Advect the level set from it's current time, time0, to it's final time, time1.
-    /// @return number of CFL iterations
+    /// Advect the level set from it's current time, time0, to it's
+    /// final time, time1. If time0>time1 backward advection is performed.
+    ///
+    /// @return number of CFL iterations used to advect from time0 to time1
     size_t advect(ScalarType time0, ScalarType time1);
 
 private:
 
+    // This templated private class implements all the level set magic.
     template<typename MapT, math::BiasedGradientScheme SpatialScheme,
              math::TemporalIntegrationScheme TemporalScheme>
     class LevelSetAdvect
@@ -270,8 +275,8 @@ private:
         /// Sample field and return the CFT time step
         typename GridT::ValueType sampleField(ScalarType time0, ScalarType time1);
         void  clearField();
-        void  sampleXformedField(const RangeType& r, ScalarType t);
-        void  sampleAlignedField(const RangeType& r, ScalarType t);
+        void  sampleXformedField(const RangeType& r, ScalarType time0, ScalarType time1);
+        void  sampleAlignedField(const RangeType& r, ScalarType time0, ScalarType time1);
         // Forward Euler advection steps: Phi(result) = Phi(0) - dt * V.Grad(0);
         void euler1(const RangeType& r, ScalarType dt, Index resultBuffer);
         // Convex combination of Phi and a forward Euler advection steps:
@@ -461,13 +466,15 @@ LevelSetAdvection<GridT, FieldT, InterruptT>::
 LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
 advect(ScalarType time0, ScalarType time1)
 {
-    size_t count = 0;
-    while (time1 > time0 && mParent.mTracker.checkInterrupter()) {
+    size_t countCFL = 0;
+    if ( math::isZero(time0 - time1) ) return countCFL;
+    const bool isForward = time0 < time1;
+    while ((isForward ? time0<time1 : time0>time1) && mParent.mTracker.checkInterrupter()) {
         /// Make sure we have enough temporal auxiliary buffers
         mParent.mTracker.mLeafs->rebuildAuxBuffers(TemporalScheme == math::TVD_RK3 ? 2 : 1);
 
         const ScalarType dt = this->sampleField(time0, time1);
-        if (math::isExactlyEqual(dt, ScalarType(0.0))) break;//V is essentially zero so terminate
+        if ( math::isZero(dt) ) break;//V is essentially zero so terminate
 
         switch(TemporalScheme) {//switch is resolved at compile-time
         case math::TVD_RK1:
@@ -512,16 +519,14 @@ advect(ScalarType time0, ScalarType time1)
         default:
             OPENVDB_THROW(ValueError, "Temporal integration scheme not supported!");
         }
-
-        time0 += dt;
-        ++count;
+        time0 += isForward ? dt : -dt;
+        ++countCFL;
         mParent.mTracker.mLeafs->removeAuxBuffers();
         this->clearField();
-
         /// Track the narrow band
         mParent.mTracker.track();
-    }
-    return count;
+    }//end wile-loop over time
+    return countCFL;//number of CLF propagation steps
 }
 
 template<typename GridT, typename FieldT, typename InterruptT>
@@ -533,18 +538,21 @@ LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
 sampleField(ScalarType time0, ScalarType time1)
 {
     mMaxAbsV = mMinAbsV;
-    if (mParent.mTracker.mLeafs->leafCount()==0) return ScalarType(0.0);
-    mVec = new VectorType*[mParent.mTracker.mLeafs->leafCount()];
+    const size_t leafCount = mParent.mTracker.mLeafs->leafCount();
+    if (leafCount==0) return ScalarType(0.0);
+    mVec = new VectorType*[leafCount];
     if (mParent.mField.transform() == mParent.mTracker.mGrid.transform()) {
-        mTask = boost::bind(&LevelSetAdvect::sampleAlignedField, _1, _2, time0);
+        mTask = boost::bind(&LevelSetAdvect::sampleAlignedField, _1, _2, time0, time1);
     } else {
-        mTask = boost::bind(&LevelSetAdvect::sampleXformedField, _1, _2, time0);
+        mTask = boost::bind(&LevelSetAdvect::sampleXformedField, _1, _2, time0, time1);
     }
     this->cook(PARALLEL_REDUCE);
     if (math::isExactlyEqual(mMinAbsV, mMaxAbsV)) return ScalarType(0.0);//V is essentially zero
     static const ScalarType CFL = (TemporalScheme == math::TVD_RK1 ? ScalarType(0.3) :
-        TemporalScheme == math::TVD_RK2 ? ScalarType(0.9) : ScalarType(1.0))/math::Sqrt(ScalarType(3.0));
-    return math::Min(time1-time0, ScalarType(CFL*mParent.mTracker.voxelSize()/math::Sqrt(mMaxAbsV)));
+                                   TemporalScheme == math::TVD_RK2 ? ScalarType(0.9) :
+                                   ScalarType(1.0))/math::Sqrt(ScalarType(3.0));
+    const ScalarType dt = math::Abs(time1 - time0), dx = mParent.mTracker.voxelSize();
+    return math::Min(dt, ScalarType(CFL*dx/math::Sqrt(mMaxAbsV)));
 }
 
 template<typename GridT, typename FieldT, typename InterruptT>
@@ -553,8 +561,9 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-sampleXformedField(const RangeType& range, ScalarType time)
+sampleXformedField(const RangeType& range, ScalarType time0, ScalarType time1)
 {
+    const bool isForward = time0 < time1;
     typedef typename LeafType::ValueOnCIter VoxelIterT;
     const MapT& map = mMap;
     mParent.mTracker.checkInterrupter();
@@ -563,9 +572,9 @@ sampleXformedField(const RangeType& range, ScalarType time)
         VectorType* vec = new VectorType[leaf.onVoxelCount()];
         int m = 0;
         for (VoxelIterT iter = leaf.cbeginValueOn(); iter; ++iter, ++m) {
-            const VectorType V = mParent.mField(map.applyMap(iter.getCoord().asVec3d()), time);
+            const VectorType V = mParent.mField(map.applyMap(iter.getCoord().asVec3d()), time0);
             mMaxAbsV = math::Max(mMaxAbsV, ScalarType(math::Pow2(V[0])+math::Pow2(V[1])+math::Pow2(V[2])));
-            vec[m] = V;
+            vec[m] = isForward ? V : -V;
         }
         mVec[n] = vec;
     }
@@ -577,8 +586,9 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
 LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-sampleAlignedField(const RangeType& range, ScalarType time)
+sampleAlignedField(const RangeType& range, ScalarType time0, ScalarType time1)
 {
+    const bool isForward = time0 < time1;
     typedef typename LeafType::ValueOnCIter VoxelIterT;
     mParent.mTracker.checkInterrupter();
     for (size_t n=range.begin(), e=range.end(); n != e; ++n) {
@@ -586,9 +596,9 @@ sampleAlignedField(const RangeType& range, ScalarType time)
         VectorType* vec = new VectorType[leaf.onVoxelCount()];
         int m = 0;
         for (VoxelIterT iter = leaf.cbeginValueOn(); iter; ++iter, ++m) {
-            const VectorType V = mParent.mField(iter.getCoord(), time);
+            const VectorType V = mParent.mField(iter.getCoord(), time0);
             mMaxAbsV = math::Max(mMaxAbsV, ScalarType(math::Pow2(V[0])+math::Pow2(V[1])+math::Pow2(V[2])));
-            vec[m] = V;
+            vec[m] = isForward ? V : -V;
         }
         mVec[n] = vec;
     }
@@ -697,6 +707,6 @@ euler2(const RangeType& range, ScalarType dt, ScalarType alpha, Index phiBuffer,
 
 #endif // OPENVDB_TOOLS_LEVEL_SET_ADVECT_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
