@@ -120,10 +120,10 @@ levelSetRebuild(const GridType& grid, float isovalue, float exBandWidth, float i
 
 namespace internal {
 
-class PointTransform
+class PointListTransform
 {
 public:
-    PointTransform(const PointList& pointsIn, std::vector<Vec3s>& pointsOut,
+    PointListTransform(const PointList& pointsIn, std::vector<Vec3s>& pointsOut,
         const math::Transform& xform)
         : mPointsIn(pointsIn)
         , mPointsOut(&pointsOut)
@@ -243,7 +243,7 @@ doLevelSetRebuild(const GridType& grid, typename GridType::ValueType iso,
     std::vector<Vec3s> points(mesher.pointListSize());
 
     { // Copy and transform (required for MeshToVolume) points to grid space.
-        internal::PointTransform ptnXForm(mesher.pointList(), points, *transform);
+        internal::PointListTransform ptnXForm(mesher.pointList(), points, *transform);
         ptnXForm.runParallel();
         mesher.pointList().reset(NULL);
     }
@@ -324,12 +324,12 @@ levelSetRebuild(const GridType& grid, float iso, float exWidth, float inWidth,
 
 template<class GridType>
 inline typename GridType::Ptr
-levelSetRebuild(const GridType& grid, float iso, float half, const math::Transform* xform)
+levelSetRebuild(const GridType& grid, float iso, float halfVal, const math::Transform* xform)
 {
     typedef typename GridType::ValueType ValueT;
     ValueT
         isovalue(zeroVal<ValueT>() + iso),
-        halfWidth(zeroVal<ValueT>() + half);
+        halfWidth(zeroVal<ValueT>() + halfVal);
 
     return doLevelSetRebuild<GridType, util::NullInterrupter>(
         grid, isovalue, halfWidth, halfWidth, xform, NULL);
