@@ -32,27 +32,63 @@
 #define OPENVDB_TOOLS_VOLUME_TO_MESH_HAS_BEEN_INCLUDED
 
 #include <openvdb/tree/ValueAccessor.h>
-#include <openvdb/tools/ValueTransformer.h>
-#include <openvdb/util/Util.h>
-#include <openvdb/math/Operators.h>
+#include <openvdb/util/Util.h> // for COORD_OFFSETS
+#include <openvdb/math/Operators.h> // for ISGradient
+#include <openvdb/tools/Morphology.h> // for dilateVoxels()
 
-#include <openvdb/tools/Morphology.h>
 #include <boost/scoped_array.hpp>
-#include <boost/intrusive/slist.hpp>
+#include <boost/scoped_ptr.hpp>
 
-#include <tbb/mutex.h>
-#include <tbb/tick_count.h>
 #include <tbb/blocked_range.h>
+#include <tbb/parallel_for.h>
+#include <tbb/parallel_reduce.h>
 
 #include <vector>
 
-#include <boost/scoped_ptr.hpp>
-
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 namespace tools {
 
+////////////////////////////////////////
+
+
+// Wrapper functions for the VolumeToMesh converter
+
+
+/// @brief Uniformly mesh any scalar grid that has a continuous isosurface.
+///
+/// @param grid     a scalar grid to mesh
+/// @param points   output list of world space points
+/// @param quads    output quad index list
+/// @param isovalue determines which isosurface to mesh
+template<typename GridType>
+void
+volumeToMesh(
+    const GridType& grid,
+    std::vector<Vec3s>& points,
+    std::vector<Vec4I>& quads,
+    double isovalue = 0.0);
+
+
+/// @brief Adaptively mesh any scalar grid that has a continuous isosurface.
+///
+/// @param grid         a scalar grid to mesh
+/// @param points       output list of world space points
+/// @param triangles    output quad index list
+/// @param quads        output quad index list
+/// @param isovalue     determines which isosurface to mesh
+/// @param adaptivity   surface adaptivity threshold [0 to 1]
+template<typename GridType>
+void
+volumeToMesh(
+    const GridType& grid,
+    std::vector<Vec3s>& points,
+    std::vector<Vec3I>& triangles,
+    std::vector<Vec4I>& quads,
+    double isovalue = 0.0,
+    double adaptivity = 0.0);
+
 
 ////////////////////////////////////////
 
@@ -140,11 +176,10 @@ typedef boost::scoped_array<PolygonPool> PolygonPoolList;
 ////////////////////////////////////////
 
 
-/// @brief Mesh any scalar grid with a continuous isosurface.
+/// @brief Mesh any scalar grid that has a continuous isosurface.
 class VolumeToMesh
 {
 public:
-
     /// @param isovalue         Determines which isosurface to mesh.
     /// @param adaptivity       Adaptivity threshold [0 to 1]
     VolumeToMesh(double isovalue = 0, double adaptivity = 0);
@@ -156,8 +191,8 @@ public:
     const PolygonPoolList& polygonPoolList() const;
     const size_t& polygonPoolListSize() const;
 
-    /// @brief main call.
-    /// @note call with scalar typed grid.
+    /// @brief main call
+    /// @note Call with scalar typed grid.
     template<typename GridT>
     void operator()(const GridT&);
 
@@ -183,11 +218,9 @@ public:
     ///                         that do not exist in the reference grid. (Parts of the
     ///                         fragment surface that are not coincident with the
     ///                         reference surface.)
-    void setRefGrid(const GridBase::ConstPtr& grid, double secAdaptivity = 0);
-
-    // Deprecated in versions 1.1
-    template<typename GridT>
-    OPENVDB_DEPRECATED void accumulateAuxiliaryData(const GridT&) { return; }
+    /// @param  smoothSeams     toggle to smooth seam line edges during mesh extraction,
+    ///                         removes staircase artifacts.
+    void setRefGrid(const GridBase::ConstPtr& grid, double secAdaptivity = 0, bool smoothSeams = true);
 
 private:
 
@@ -198,8 +231,8 @@ private:
     double mIsovalue, mPrimAdaptivity, mSecAdaptivity;
 
     GridBase::ConstPtr mRefGrid;
-    TreeBase::Ptr mRefEdgeTree, mRefTopologyMaskTree;
-    bool mRefDataCached;
+    TreeBase::Ptr mRefEdgeTree, mRefTopologyMaskTree, mSeamPointTree;
+    bool mSmoothSeams;
 };
 
 
@@ -551,12 +584,15 @@ struct ReferenceData
     typedef typename DistTreeT::ValueType DistValueT;
     typedef typename DistTreeT::template ValueConverter<char>::Type CharTreeT;
     typedef typename DistTreeT::template ValueConverter<bool>::Type BoolTreeT;
+    typedef typename DistTreeT::template ValueConverter<Vec3s>::Type Vec3sTreeT;
 
     ReferenceData()
         : mDistTree(NULL)
         , mEdgeTree(NULL)
         , mTopologyMaskTree(NULL)
+        , mSeamPointTree(NULL)
         , mSeamMaskTree(typename BoolTreeT::Ptr())
+        , mSmoothingMaskTree(typename BoolTreeT::Ptr())
         , mInternalAdaptivity(DistValueT(0.0))
     {
     }
@@ -568,8 +604,9 @@ struct ReferenceData
 
     const DistTreeT* mDistTree;
     const CharTreeT* mEdgeTree;
-    const BoolTreeT* mTopologyMaskTree;
-    typename BoolTreeT::Ptr mSeamMaskTree;
+    BoolTreeT* mTopologyMaskTree;
+    Vec3sTreeT* mSeamPointTree;
+    typename BoolTreeT::Ptr mSeamMaskTree, mSmoothingMaskTree;
     DistValueT mInternalAdaptivity;
 };
 
@@ -1003,19 +1040,24 @@ void
 PointGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
 {
     typedef typename DistTreeT::template ValueConverter<bool>::Type BoolTreeT;
+    typedef typename DistTreeT::template ValueConverter<Vec3s>::Type Vec3sTreeT;
 
-    typedef tree::ValueAccessor<const BoolTreeT> BoolTreeAccessorT;
+    typedef tree::ValueAccessor<BoolTreeT> BoolTreeAccessorT;
+    typedef tree::ValueAccessor<Vec3sTreeT> Vec3sTreeAccessorT;
 
     typedef typename BoolTreeT::LeafNodeType BoolLeafT;
     typedef typename IntTreeT::LeafNodeType IntLeafT;
-
+    typedef typename Vec3sTreeT::LeafNodeType Vec3sLeafT;
 
     boost::scoped_ptr<DistTreeAccessorT> refDistAcc;
-    boost::scoped_ptr<BoolTreeAccessorT> refMaskAcc;
+    boost::scoped_ptr<BoolTreeAccessorT> refMaskAcc, refSmoothMaskAcc;
+    boost::scoped_ptr<Vec3sTreeAccessorT> refPtnAcc;
 
     if (mRefData && mRefData->isValid()) {
         refDistAcc.reset(new DistTreeAccessorT(*mRefData->mDistTree));
         refMaskAcc.reset(new BoolTreeAccessorT(*mRefData->mTopologyMaskTree));
+        refSmoothMaskAcc.reset(new BoolTreeAccessorT(*mRefData->mSmoothingMaskTree));
+        refPtnAcc.reset(new Vec3sTreeAccessorT(*mRefData->mSeamPointTree));
     }
 
 
@@ -1031,9 +1073,13 @@ PointGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
         size_t idx = mLeafIndices[n];
         IntLeafT& auxLeaf = *mAuxLeafs[n];
 
-        const BoolLeafT* maskLeaf = NULL;
+        BoolLeafT* maskLeaf = NULL;
+        BoolLeafT* smoothMaskLeaf = NULL;
+        Vec3sLeafT* ptnLeaf = NULL;
         if (hasRefData) {
-            maskLeaf = refMaskAcc->probeConstLeaf(auxLeaf.getOrigin());
+            maskLeaf = refMaskAcc->probeLeaf(auxLeaf.getOrigin());
+            smoothMaskLeaf = refSmoothMaskAcc->probeLeaf(auxLeaf.getOrigin());
+            ptnLeaf = refPtnAcc->probeLeaf(auxLeaf.getOrigin());
         }
 
         for (auxIter = auxLeaf.beginValueOn(); auxIter; ++auxIter) {
@@ -1045,18 +1091,21 @@ PointGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
                 ijk = auxIter.getCoord();
 
                 if (hasRefData && maskLeaf && maskLeaf->isValueOn(ijk)) {
-                    calcAvgPoint(*refDistAcc.get(), ijk, avg);
+
+                    if (ptnLeaf && ptnLeaf->isValueOn(ijk)) {
+                        avg = ptnLeaf->getValue(ijk);
+                    } else {
+                        int e1 = calcAvgPoint(*refDistAcc.get(), ijk, avg);
+
+                        if (e1 != (XEDGE|YEDGE|ZEDGE)) {
+                            int e2 = calcAvgPoint(distAcc, ijk, tmp);
+                            if((e2 & (~e1)) != 0) smoothMaskLeaf->setValueOn(ijk);
+                        }
+                    }
                 } else {
                     calcAvgPoint(distAcc, ijk, avg);
                 }
 
-                // offset by cell-origin
-                avg[0] += double(ijk[0]);
-                avg[1] += double(ijk[1]);
-                avg[2] += double(ijk[2]);
-
-                avg = mTransform.indexToWorld(avg);
-
                 openvdb::Vec3s& ptn = mPoints[idx];
                 ptn[0] = float(avg[0]);
                 ptn[1] = float(avg[1]);
@@ -1088,12 +1137,7 @@ PointGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
                         calcAvgPoint(distAcc, ijk, tmp);
                     }
 
-                    // offset by cell-origin
-                    tmp[0] += double(ijk[0]);
-                    tmp[1] += double(ijk[1]);
-                    tmp[2] += double(ijk[2]);
-
-                    avg += mTransform.indexToWorld(tmp);
+                    avg += tmp;
                     ++points;
                 }
             }
@@ -1247,6 +1291,13 @@ PointGen<DistTreeT>::calcAvgPoint(DistTreeAccessorT& acc,
         avg[2] *= w;
     }
 
+    // offset by cell-origin
+    avg[0] += double(ijk[0]);
+    avg[1] += double(ijk[1]);
+    avg[2] += double(ijk[2]);
+
+    avg = mTransform.indexToWorld(avg);
+
     return edgeFlags;
 }
 
@@ -1591,7 +1642,7 @@ MeshGen<DistTreeT, MeshingOp>::operator()(
                 quad[3] = auxAcc.getValue(coord);
 
                 mesher.addPrim(quad, !isInside,
-                    (isSemLinePoly | isExteriorPoly[bool(refEdgeFlags & ZEDGE)]));
+                     (isSemLinePoly | isExteriorPoly[bool(refEdgeFlags & ZEDGE)]));
             }
         }
 
@@ -1931,6 +1982,149 @@ SeamMaskGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) cons
     }
 }
 
+////////////////////////////////////////
+
+
+template <class DistTreeT>
+class EdgeSmooth
+{
+public:
+    typedef typename DistTreeT::template ValueConverter<bool>::Type BoolTreeT;
+    typedef typename DistTreeT::template ValueConverter<int>::Type  IntTreeT;
+    typedef tree::ValueAccessor<const BoolTreeT> BoolTreeAccessorT;
+    typedef tree::ValueAccessor<const IntTreeT> IntTreeAccessorT;
+
+    EdgeSmooth(
+        LeafPtrList<BoolTreeT>& leafs,
+        const BoolTreeT& edgeMaskTree,
+        const IntTreeT& auxTree,
+        PointList& points,
+        const math::Transform& xform);
+
+    EdgeSmooth(const EdgeSmooth<DistTreeT>&);
+
+    void runParallel(const size_t iterations);
+    void runSerial(const size_t iterations);
+
+    void operator()(const tbb::blocked_range<size_t>&) const;
+
+private:
+    LeafPtrList<BoolTreeT>& mLeafs;
+    const BoolTreeT& mEdgeMaskTree;
+    const IntTreeT& mAuxTree;
+    PointList& mPoints;
+    const math::Transform& mTransform;
+
+    bool pointInAABB(const Vec3s& p, const Vec3s& bmin, const Vec3s& bmax) const
+    {
+        for (int i = 0; i < 3; ++i) {
+            if (p[i] < bmin[i] || p[i] > bmax[i]) {
+                return false;
+            }
+        }
+        return true;
+    }
+
+};
+
+
+template <class DistTreeT>
+EdgeSmooth<DistTreeT>::EdgeSmooth(
+    LeafPtrList<BoolTreeT>& leafs,
+    const BoolTreeT& edgeMaskTree,
+    const IntTreeT& auxTree,
+    PointList& points,
+    const math::Transform& xform)
+    : mLeafs(leafs)
+    , mEdgeMaskTree(edgeMaskTree)
+    , mAuxTree(auxTree)
+    , mPoints(points)
+    , mTransform(xform)
+{
+}
+
+template <class DistTreeT>
+EdgeSmooth<DistTreeT>::EdgeSmooth(const EdgeSmooth<DistTreeT>& rhs)
+    : mLeafs(rhs.mLeafs)
+    , mEdgeMaskTree(rhs.mEdgeMaskTree)
+    , mAuxTree(rhs.mAuxTree)
+    , mPoints(rhs.mPoints)
+    , mTransform(rhs.mTransform)
+{
+}
+
+template <class DistTreeT>
+void
+EdgeSmooth<DistTreeT>::runParallel(const size_t iterations)
+{
+    for (size_t i = 0; i < iterations; ++i) {
+        tbb::parallel_for(mLeafs.getRange(), *this);
+    }
+}
+
+template <class DistTreeT>
+void
+EdgeSmooth<DistTreeT>::runSerial(const size_t iterations)
+{
+    for (size_t i = 0; i < iterations; ++i) {
+        (*this)(mLeafs.getRange());
+    }
+}
+
+template <class DistTreeT>
+void
+EdgeSmooth<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
+{
+    typedef typename BoolTreeT::LeafNodeType::ValueOnIter ValueOnIterT;
+    BoolTreeAccessorT maskAcc(mEdgeMaskTree);
+    IntTreeAccessorT auxAcc(mAuxTree);
+
+    float dx = float(mTransform.voxelSize()[0]);
+    openvdb::Vec3s avg, bmin, bmax;
+    Coord ijk, nijk;
+    int count;
+
+    for (size_t leafIdx = range.begin(); leafIdx != range.end(); ++leafIdx) {
+        typename BoolTreeT::LeafNodeType::ValueOnIter valueIt = mLeafs[leafIdx]->beginValueOn();
+        for ( ; valueIt; ++valueIt) {
+
+            ijk = valueIt.getCoord();
+            openvdb::Vec3s& ptn = mPoints[auxAcc.getValue(ijk)];
+
+            avg = ptn;
+            count = 1;
+            for (int n = 0; n < 26; ++n) {
+                nijk = ijk + util::COORD_OFFSETS[n];
+                if (maskAcc.isValueOn(nijk)) {
+                    avg += mPoints[auxAcc.getValue(nijk)];
+                    ++count;
+                }
+            }
+
+            if (count > 1) {
+                avg *= (1.0 / float(count));
+
+                // Constrain to current cell
+                bmin = openvdb::Vec3s(mTransform.indexToWorld(ijk));
+                bmax = bmin + dx;
+
+                bool inCell = true;
+                for (int i = 0; i < 10; ++i) {
+
+                    inCell = pointInAABB(avg, bmin, bmax);
+
+                    if (inCell) break;
+
+                    avg += ptn;
+                    avg *= 0.5;
+                }
+
+                if (inCell) ptn = avg;
+            }
+        }
+    }
+}
+
 
 ////////////////////////////////////////
 
@@ -2169,6 +2363,31 @@ tileAuxiliaryData(
 }
 
 
+////////////////////////////////////////
+
+
+// Utility class for the volumeToMesh wrapper
+class PointListCopy
+{
+public:
+    PointListCopy(const PointList& pointsIn, std::vector<Vec3s>& pointsOut)
+        : mPointsIn(pointsIn) , mPointsOut(pointsOut)
+    {
+    }
+
+    void operator()(const tbb::blocked_range<size_t>& range) const
+    {
+        for (size_t n = range.begin(); n < range.end(); ++n) {
+            mPointsOut[n] = mPointsIn[n];
+        }
+    }
+
+private:
+    const PointList& mPointsIn;
+    std::vector<Vec3s>& mPointsOut;
+};
+
+
 } // namespace internal
 
 
@@ -2222,13 +2441,15 @@ VolumeToMesh::polygonPoolListSize() const
 
 
 inline void
-VolumeToMesh::setRefGrid(const GridBase::ConstPtr& grid, double secAdaptivity)
+VolumeToMesh::setRefGrid(const GridBase::ConstPtr& grid, double secAdaptivity, bool smoothSeams)
 {
     mRefGrid = grid;
     mSecAdaptivity = secAdaptivity;
     // Clear out old auxiliary data
     mRefEdgeTree = TreeBase::Ptr();
     mRefTopologyMaskTree = TreeBase::Ptr();
+    mSeamPointTree = TreeBase::Ptr();
+    mSmoothSeams = smoothSeams;
 }
 
 
@@ -2241,6 +2462,7 @@ VolumeToMesh::operator()(const GridT& distGrid)
 
     typedef typename DistTreeT::template ValueConverter<char>::Type CharTreeT;
     typedef typename DistTreeT::template ValueConverter<int>::Type IntTreeT;
+    typedef typename DistTreeT::template ValueConverter<Vec3s>::Type Vec3sTreeT;
     typedef typename DistTreeT::template ValueConverter<bool>::Type BoolTreeT;
 
     const bool noAdaptivity = mPrimAdaptivity < 1e-6 && mSecAdaptivity < 1e-6;
@@ -2255,9 +2477,10 @@ VolumeToMesh::operator()(const GridT& distGrid)
     const bool extraSignCheck = nonLevelSetGrid ||
         (std::abs(mIsovalue - double(distGrid.background())) < (1.5 * transform.voxelSize()[0]));
 
+
+
     // Collect auxiliary data
     {
-
         internal::LeafCPtrList<DistTreeT> sourceLeafs(distTree);
         internal::AuxiliaryData<DistTreeT> op(distTree, sourceLeafs, mIsovalue, extraSignCheck);
         op.runParallel();
@@ -2270,6 +2493,7 @@ VolumeToMesh::operator()(const GridT& distGrid)
         }
     }
 
+
     // Optionally collect auxiliary data from a reference surface.
     internal::ReferenceData<DistTreeT> refData;
     if (mRefGrid) {
@@ -2287,16 +2511,21 @@ VolumeToMesh::operator()(const GridT& distGrid)
                 op.runParallel();
                 mRefEdgeTree = op.edgeTree();
                 mRefTopologyMaskTree = op.auxTree();
+                mSeamPointTree = typename Vec3sTreeT::Ptr(new Vec3sTreeT(Vec3s(0.0)));
             }
 
             if (mRefEdgeTree && mRefTopologyMaskTree) {
                 refData.mEdgeTree = static_cast<CharTreeT*>(mRefEdgeTree.get());
                 refData.mTopologyMaskTree =  static_cast<BoolTreeT*>(mRefTopologyMaskTree.get());
+                refData.mSeamPointTree = static_cast<Vec3sTreeT*>(mSeamPointTree.get());
                 refData.mSeamMaskTree = typename BoolTreeT::Ptr(new BoolTreeT(false));
+                refData.mSmoothingMaskTree = typename BoolTreeT::Ptr(new BoolTreeT(false));
             }
         }
     }
 
+    BoolTreeT edgeMaskTree(0.0);
+
     // Generate the seamline mask
     if (refData.mSeamMaskTree) {
         refData.mSeamMaskTree->topologyUnion(*auxTree.get());
@@ -2306,11 +2535,13 @@ VolumeToMesh::operator()(const GridT& distGrid)
         op.runParallel();
 
         refData.mSeamMaskTree->pruneInactive();
+        edgeMaskTree.topologyUnion(*refData.mSeamMaskTree);
         dilateVoxels(*refData.mSeamMaskTree);
         dilateVoxels(*refData.mSeamMaskTree);
         dilateVoxels(*refData.mSeamMaskTree);
     }
 
+
     // Process auxiliary data
     {
         internal::LeafPtrList<IntTreeT> auxLeafs(*auxTree);
@@ -2336,6 +2567,13 @@ VolumeToMesh::operator()(const GridT& distGrid)
             }
         }
 
+        if (refData.isValid()) { // match leaf topology
+            tree::ValueAccessor<BoolTreeT> acc(*refData.mSmoothingMaskTree);
+            for (size_t n = 0, N = auxLeafs.size(); n < N; ++n) {
+                acc.touchLeaf(auxLeafs[n]->getOrigin());
+            }
+        }
+
         // Generate the unique point list
         mPoints.reset(new openvdb::Vec3s[mPointListSize]);
 
@@ -2345,6 +2583,31 @@ VolumeToMesh::operator()(const GridT& distGrid)
         pointGen.runParallel();
     }
 
+    // Smooth seam line edges
+    if (mSmoothSeams && refData.isValid()) {
+        refData.mSmoothingMaskTree->pruneInactive();
+        internal::LeafPtrList<BoolTreeT> leafs(*refData.mSmoothingMaskTree);
+        internal::EdgeSmooth<DistTreeT> op(leafs, edgeMaskTree, *auxTree, mPoints, transform);
+        op.runParallel(3);
+
+        // Cache shared points
+        tree::ValueAccessor<Vec3sTreeT> ptnAcc(*refData.mSeamPointTree);
+        tree::ValueAccessor<IntTreeT> auxAcc(*auxTree);
+        Coord ijk;
+
+        typename BoolTreeT::LeafIter leafIt = refData.mSeamMaskTree->beginLeaf();
+        for ( ; leafIt; ++leafIt) {
+            typename BoolTreeT::LeafNodeType::ValueOnIter valueIt = leafIt->beginValueOn();
+            for ( ; valueIt; ++valueIt) {
+                ijk = valueIt.getCoord();
+                const int idx = auxAcc.getValue(ijk);
+                if (idx != 0 && !ptnAcc.isValueOn(ijk)) {
+                   ptnAcc.setValue(ijk, mPoints[idx]);
+                }
+            }
+        }
+    }
+
     // Generate mesh
     {
         internal::LeafCPtrList<CharTreeT> edgeLeafs(*edgeTree);
@@ -2365,6 +2628,77 @@ VolumeToMesh::operator()(const GridT& distGrid)
     }
 }
 
+
+////////////////////////////////////////
+
+
+template<typename GridType>
+inline void
+volumeToMesh(
+    const GridType& grid,
+    std::vector<Vec3s>& points,
+    std::vector<Vec3I>& triangles,
+    std::vector<Vec4I>& quads,
+    double isovalue,
+    double adaptivity)
+{
+    VolumeToMesh mesher(isovalue, adaptivity);
+    mesher(grid);
+
+    // Preallocate the point list
+    points.clear();
+    points.resize(mesher.pointListSize());
+
+    { // Copy points
+        internal::PointListCopy ptnCpy(mesher.pointList(), points);
+        tbb::parallel_for(tbb::blocked_range<size_t>(0, points.size()), ptnCpy);
+        mesher.pointList().reset(NULL);
+    }
+
+    PolygonPoolList& polygonPoolList = mesher.polygonPoolList();
+
+    { // Preallocate primitive lists
+        size_t numQuads = 0, numTriangles = 0;
+        for (size_t n = 0, N = mesher.polygonPoolListSize(); n < N; ++n) {
+            openvdb::tools::PolygonPool& polygons = polygonPoolList[n];
+            numTriangles += polygons.numTriangles();
+            numQuads += polygons.numQuads();
+        }
+
+        triangles.clear();
+        triangles.resize(numTriangles);
+        quads.clear();
+        quads.resize(numQuads);
+    }
+
+    // Copy primitives
+    size_t qIdx = 0, tIdx = 0;
+    for (size_t n = 0, N = mesher.polygonPoolListSize(); n < N; ++n) {
+        openvdb::tools::PolygonPool& polygons = polygonPoolList[n];
+
+        for (size_t i = 0, I = polygons.numQuads(); i < I; ++i) {
+            quads[qIdx++] = polygons.quad(i);
+        }
+
+        for (size_t i = 0, I = polygons.numTriangles(); i < I; ++i) {
+            triangles[tIdx++] = polygons.triangle(i);
+        }
+    }
+}
+
+
+template<typename GridType>
+void
+volumeToMesh(
+    const GridType& grid,
+    std::vector<Vec3s>& points,
+    std::vector<Vec4I>& quads,
+    double isovalue)
+{
+    std::vector<Vec3I> triangles(0);
+    volumeToMesh(grid,points, triangles, quads, isovalue, 0.0);
+}
+
 } // namespace tools
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
