@@ -291,6 +291,18 @@ public:
     /// (The min and max coordinates are inclusive.)
     void fill(const CoordBBox& bbox, const ValueType&, bool active = true);
 
+    /// @brief Copy into a dense grid the values of the voxels that lie within
+    /// a given bounding box.
+    ///
+    /// @param bbox   inclusive bounding box of the voxels to be copied into the dense grid
+    /// @param dense  dense grid with a stride in @e z of one (see tools::Dense
+    ///               in tools/Dense.h for the required API)
+    ///
+    /// @note @a bbox is assumed to be identical to or contained in the coordinate domains
+    /// of both the dense grid and this node, i.e., no bounds checking is performed.
+    template<typename DenseT>
+    void copyToDense(const CoordBBox& bbox, DenseT& dense) const;
+
     /// Return the value of the voxel at the given coordinates and, if necessary, update
     /// the accessor with pointers to the nodes along the path from the root node to
     /// the node containing the voxel.
@@ -470,6 +482,60 @@ public:
     /// background tiles any nodes whose values are all inactive.
     void pruneInactive();
 
+    /// @brief Add the specified leaf to this node, possibly creating a child branch
+    /// in the process.  If the leaf node already exists, replace it.
+    void addLeaf(LeafNodeType* leaf);
+
+    /// @brief Same as addLeaf except, if necessary, it update the accessor with pointers
+    /// to the nodes along the path from the root node to the node containing the coordinate.
+    template<typename AccessorT>
+    void addLeafAndCache(LeafNodeType* leaf, AccessorT&);
+
+    /// @brief Return a pointer to the node of type @c NodeT that contains voxel (x, y, z)
+    /// and replace it with a tile of the specified value and state.
+    /// If no such node exists, leave the tree unchanged and return @c NULL.
+    ///
+    /// @note The caller takes ownership of the node and is responsible for deleting it.
+    ///
+    /// @warning Since this method potentially removes nodes and branches of the tree,
+    /// it is important to clear the caches of all ValueAccessors associated with this tree.
+    template<typename NodeT>
+    NodeT* stealNode(const Coord& xyz, const ValueType& value, bool state);
+
+    /// @brief Add a tile at the specified tree level that contains voxel (x, y, z),
+    /// possibly creating a parent branch or deleting a child branch in the process.
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool state);
+
+    /// @brief Same as addTile() except, if necessary, update the accessor with pointers
+    /// to the nodes along the path from the root node to the node containing (x, y, z).
+    template<typename AccessorT>
+    void addTileAndCache(Index level, const Coord& xyz, const ValueType& value,
+        bool state, AccessorT&);
+
+    /// @brief Return a pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, return NULL.
+    LeafNodeType* probeLeaf(const Coord& xyz);
+
+    /// @brief Return a const pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, return NULL.
+    const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
+    const LeafNodeType* probeLeaf(const Coord& xyz) const { return this->probeConstLeaf(xyz); }
+
+    /// @brief Same as probeLeaf except, if necessary, it update the accessor with pointers
+    /// to the nodes along the path from the root node to the node containing the coordinate.
+    template<typename AccessorT>
+    LeafNodeType* probeLeafAndCache(const Coord& xyz, AccessorT&);
+
+    /// @brief Same as probeLeaf except, if necessary, it update the accessor with pointers
+    /// to the nodes along the path from the root node to the node containing the coordinate.
+    template<typename AccessorT>
+    const LeafNodeType* probeConstLeafAndCache(const Coord& xyz, AccessorT& acc) const;
+    template<typename AccessorT>
+    const LeafNodeType* probeLeafAndCache(const Coord& xyz, AccessorT& acc) const
+    {
+        return this->probeConstLeafAndCache(xyz, acc);
+    }
+
     /// @brief Return the leaf node that contains voxel (x, y, z).
     /// If no such node exists, create one, but preserve the values and
     /// active states of all voxels.
@@ -478,27 +544,11 @@ public:
     /// over which to safely perform multithreaded processing.
     LeafNodeType* touchLeaf(const Coord& xyz);
 
-    /// @brief Return a pointer to the leaf node that contains voxel (x, y, z).
-    /// If no such node exists, return NULL.
-    LeafNodeType* probeLeaf(const Coord& xyz);
-    /// @brief Return a const pointer to the leaf node that contains voxel (x, y, z).
-    /// If no such node exists, return NULL.
-    const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
-
     /// @brief Same as touchLeaf except, if necessary, it update the accessor with pointers
     /// to the nodes along the path from the root node to the node containing the coordinate.
     template<typename AccessorT>
     LeafNodeType* touchLeafAndCache(const Coord& xyz, AccessorT&);
 
-    /// @brief Same as probeLeaf except, if necessary, it update the accessor with pointers
-    /// to the nodes along the path from the root node to the node containing the coordinate.
-    template<typename AccessorT>
-    LeafNodeType* probeLeafAndCache(const Coord& xyz, AccessorT&);
-    /// @brief Same as probeConstLeaf except, if necessary, it update the accessor with pointers
-    /// to the nodes along the path from the root node to the node containing the coordinate.
-    template<typename AccessorT>
-    const LeafNodeType* probeConstLeafAndCache(const Coord& xyz, AccessorT&) const;
-
     /// @brief Change inactive tiles or voxels with value oldBackground to newBackground
     /// or -oldBackground to -newBackground. Active values are unchanged.
     void resetBackground(const ValueType& oldBackground, const ValueType& newBackground);
@@ -541,6 +591,9 @@ protected:
     void setChildNode(Index i, ChildNodeType* child);
     ChildNodeType* unsetChildNode(Index i, const ValueType& value);
 
+    template<typename NodeT>
+    NodeT* doStealNode(const Coord& xyz, const ValueType& value, bool state);
+
     template<typename NodeT, typename VisitorOp, typename ChildAllIterT>
     static inline void doVisit(NodeT&, VisitorOp&);
 
@@ -809,19 +862,209 @@ InternalNode<ChildT, Log2Dim>::pruneInactive()
     this->pruneInactive(this->getBackground());
 }
 
+
+////////////////////////////////////////
+
+
+// Helper method that implements stealNode()
+template<typename ChildT, Index Log2Dim>
+template<typename NodeT>
+inline NodeT*
+InternalNode<ChildT, Log2Dim>::doStealNode(const Coord& xyz, const ValueType& value, bool state)
+{
+    const Index n = this->coord2offset(xyz);
+    if (mChildMask.isOff(n)) return NULL;
+    ChildT* child = mNodes[n].getChild();
+    if (boost::is_same<NodeT, ChildT>::value) {
+        mChildMask.setOff(n);
+        mValueMask.set(n, state);
+        mNodes[n].setValue(value);
+    }
+    return (boost::is_same<NodeT, ChildT>::value)
+        ? reinterpret_cast<NodeT*>(child)
+        : child->template stealNode<NodeT>(xyz, value, state);
+}
+
+
+template<typename ChildT, Index Log2Dim>
+template<typename NodeT>
+inline NodeT*
+InternalNode<ChildT, Log2Dim>::stealNode(const Coord& xyz, const ValueType& value, bool state)
+{
+    // The following conditional is resolved at compile time, and the ternary operator
+    // and helper method are used to avoid "unreachable code" warnings (with
+    // "if (<cond>) { <A> } else { <B> }", either <A> or <B> is unreachable if <cond>
+    // is a compile-time constant expression).  Partial specialization on NodeT would be
+    // impractical because a method template can't be specialized without also
+    // specializing its class template.
+    return (NodeT::LEVEL > ChildT::LEVEL ||
+        (NodeT::LEVEL == ChildT::LEVEL && !(boost::is_same<NodeT, ChildT>::value)))
+        ? static_cast<NodeT*>(NULL)
+        : this->doStealNode<NodeT>(xyz, value, state);
+}
+
+
+////////////////////////////////////////
+
+
+template<typename ChildT, Index Log2Dim>
+inline void
+InternalNode<ChildT, Log2Dim>::addLeaf(LeafNodeType* leaf)
+{
+    assert(leaf != NULL);
+    const Coord& xyz = leaf->origin();
+    const Index n = this->coord2offset(xyz);
+    ChildT* child = NULL;
+    if (mChildMask.isOff(n)) {
+        if (ChildT::LEVEL>0) {
+            child = new ChildT(xyz, mNodes[n].getValue(), mValueMask.isOn(n));
+        } else {
+            child = reinterpret_cast<ChildT*>(leaf);
+        }
+        mNodes[n].setChild(child);
+        mChildMask.setOn(n);
+        mValueMask.setOff(n);
+    } else {
+        if (ChildT::LEVEL>0) {
+            child = mNodes[n].getChild();
+        } else {
+            delete mNodes[n].getChild();
+            child = reinterpret_cast<ChildT*>(leaf);
+            mNodes[n].setChild(child);
+        }
+    }
+    child->addLeaf(leaf);
+}
+
+
+template<typename ChildT, Index Log2Dim>
+template<typename AccessorT>
+inline void
+InternalNode<ChildT, Log2Dim>::addLeafAndCache(LeafNodeType* leaf, AccessorT& acc)
+{
+    assert(leaf != NULL);
+    const Coord& xyz = leaf->origin();
+    const Index n = this->coord2offset(xyz);
+    ChildT* child = NULL;
+    if (mChildMask.isOff(n)) {
+        if (ChildT::LEVEL>0) {
+            child = new ChildT(xyz, mNodes[n].getValue(), mValueMask.isOn(n));
+            acc.insert(xyz, child);//we only cache internal nodes
+        } else {
+            child = reinterpret_cast<ChildT*>(leaf);
+        }
+        mNodes[n].setChild(child);
+        mChildMask.setOn(n);
+        mValueMask.setOff(n);
+    } else {
+        if (ChildT::LEVEL>0) {
+            child = mNodes[n].getChild();
+            acc.insert(xyz, child);//we only cache internal nodes
+        } else {
+            delete mNodes[n].getChild();
+            child = reinterpret_cast<ChildT*>(leaf);
+            mNodes[n].setChild(child);
+        }
+    }
+    child->addLeafAndCache(leaf, acc);
+}
+
+
+////////////////////////////////////////
+
+
+template<typename ChildT, Index Log2Dim>
+inline void
+InternalNode<ChildT, Log2Dim>::addTile(Index level, const Coord& xyz,
+                                       const ValueType& value, bool state)
+{
+    assert(level > 0);
+    if (LEVEL >= level) {
+        const Index n = this->coord2offset(xyz);
+        if (mChildMask.isOff(n)) {// tile case
+            if (LEVEL > level) {
+                ChildT* child = new ChildT(xyz, mNodes[n].getValue(), mValueMask.isOn(n));
+                mNodes[n].setChild(child);
+                mChildMask.setOn(n);
+                mValueMask.setOff(n);
+                child->addTile(level, xyz, value, state);
+            } else {
+                mValueMask.set(n, state);
+                mNodes[n].setValue(value);
+            }
+        } else {// child branch case
+            ChildT* child = mNodes[n].getChild();
+            if (LEVEL > level) {
+                child->addTile(level, xyz, value, state);
+            } else {
+                delete child;
+                mChildMask.setOff(n);
+                mValueMask.set(n, state);
+                mNodes[n].setValue(value);
+            }
+        }
+    }
+}
+
+
+template<typename ChildT, Index Log2Dim>
+template<typename AccessorT>
+inline void
+InternalNode<ChildT, Log2Dim>::addTileAndCache(Index level, const Coord& xyz,
+    const ValueType& value, bool state, AccessorT& acc)
+{
+    assert(level > 0);
+    if (LEVEL >= level) {
+        const Index n = this->coord2offset(xyz);
+        if (mChildMask.isOff(n)) {// tile case
+            if (LEVEL > level) {
+                ChildT* child = new ChildT(xyz, mNodes[n].getValue(), mValueMask.isOn(n));
+                mNodes[n].setChild(child);
+                mChildMask.setOn(n);
+                mValueMask.setOff(n);
+                acc.insert(xyz, child);
+                child->addTileAndCache(level, xyz, value, state, acc);
+            } else {
+                mValueMask.set(n, state);
+                mNodes[n].setValue(value);
+            }
+        } else {// child branch case
+            ChildT* child = mNodes[n].getChild();
+            if (LEVEL > level) {
+                acc.insert(xyz, child);
+                child->addTileAndCache(level, xyz, value, state, acc);
+            } else {
+                delete child;
+                mChildMask.setOff(n);
+                mValueMask.set(n, state);
+                mNodes[n].setValue(value);
+            }
+        }
+    }
+}
+
+
+////////////////////////////////////////
+
+
 template<typename ChildT, Index Log2Dim>
 inline typename ChildT::LeafNodeType*
 InternalNode<ChildT, Log2Dim>::touchLeaf(const Coord& xyz)
 {
     const Index n = this->coord2offset(xyz);
+    ChildT* child = NULL;
     if (mChildMask.isOff(n)) {
-        mNodes[n].setChild(new ChildNodeType(xyz, mNodes[n].getValue(), mValueMask.isOn(n)));
+        child = new ChildT(xyz, mNodes[n].getValue(), mValueMask.isOn(n));
+        mNodes[n].setChild(child);
         mChildMask.setOn(n);
         mValueMask.setOff(n);
+    } else {
+        child = mNodes[n].getChild();
     }
-    return mNodes[n].getChild()->touchLeaf(xyz);
+    return child->touchLeaf(xyz);
 }
 
+
 template<typename ChildT, Index Log2Dim>
 template<typename AccessorT>
 inline typename ChildT::LeafNodeType*
@@ -834,9 +1077,13 @@ InternalNode<ChildT, Log2Dim>::touchLeafAndCache(const Coord& xyz, AccessorT& ac
         mValueMask.setOff(n);
     }
     acc.insert(xyz, mNodes[n].getChild());
-    return mNodes[n].getChild()->touchLeafAndCache(xyz,acc);
+    return mNodes[n].getChild()->touchLeafAndCache(xyz, acc);
 }
 
+
+////////////////////////////////////////
+
+
 template<typename ChildT, Index Log2Dim>
 inline typename ChildT::LeafNodeType*
 InternalNode<ChildT, Log2Dim>::probeLeaf(const Coord& xyz)
@@ -845,6 +1092,7 @@ InternalNode<ChildT, Log2Dim>::probeLeaf(const Coord& xyz)
     return mChildMask.isOn(n) ? mNodes[n].getChild()->probeLeaf(xyz) : NULL;
 }
 
+
 template<typename ChildT, Index Log2Dim>
 inline const typename ChildT::LeafNodeType*
 InternalNode<ChildT, Log2Dim>::probeConstLeaf(const Coord& xyz) const
@@ -853,6 +1101,7 @@ InternalNode<ChildT, Log2Dim>::probeConstLeaf(const Coord& xyz) const
     return mChildMask.isOn(n) ? mNodes[n].getChild()->probeConstLeaf(xyz) : NULL;
 }
 
+
 template<typename ChildT, Index Log2Dim>
 template<typename AccessorT>
 inline typename ChildT::LeafNodeType*
@@ -861,9 +1110,10 @@ InternalNode<ChildT, Log2Dim>::probeLeafAndCache(const Coord& xyz, AccessorT& ac
     const Index n = this->coord2offset(xyz);
     if (mChildMask.isOff(n)) return NULL;
     acc.insert(xyz, mNodes[n].getChild());
-    return mNodes[n].getChild()->probeLeafAndCache(xyz,acc);
+    return mNodes[n].getChild()->probeLeafAndCache(xyz, acc);
 }
 
+
 template<typename ChildT, Index Log2Dim>
 template<typename AccessorT>
 inline const typename ChildT::LeafNodeType*
@@ -872,7 +1122,7 @@ InternalNode<ChildT, Log2Dim>::probeConstLeafAndCache(const Coord& xyz, Accessor
     const Index n = this->coord2offset(xyz);
     if (mChildMask.isOff(n)) return NULL;
     acc.insert(xyz, mNodes[n].getChild());
-    return mNodes[n].getChild()->probeConstLeafAndCache(xyz,acc);
+    return mNodes[n].getChild()->probeConstLeafAndCache(xyz, acc);
 }
 
 
@@ -928,15 +1178,6 @@ InternalNode<ChildT, Log2Dim>::isConstant(ValueType& constValue, bool& state,
 ////////////////////////////////////////
 
 
-template<typename ChildT, Index Log2Dim>
-inline bool
-InternalNode<ChildT, Log2Dim>::isValueOn(const Coord& xyz) const
-{
-    const Index n = this->coord2offset(xyz);
-    if (this->isChildMaskOff(n)) return this->isValueMaskOn(n);
-    return mNodes[n].getChild()->isValueOn(xyz);
-}
-
 template<typename ChildT, Index Log2Dim>
 inline bool
 InternalNode<ChildT, Log2Dim>::hasActiveTiles() const
@@ -949,6 +1190,16 @@ InternalNode<ChildT, Log2Dim>::hasActiveTiles() const
     return false;
 }
 
+
+template<typename ChildT, Index Log2Dim>
+inline bool
+InternalNode<ChildT, Log2Dim>::isValueOn(const Coord& xyz) const
+{
+    const Index n = this->coord2offset(xyz);
+    if (this->isChildMaskOff(n)) return this->isValueMaskOn(n);
+    return mNodes[n].getChild()->isValueOn(xyz);
+}
+
 template<typename ChildT, Index Log2Dim>
 template<typename AccessorT>
 inline bool
@@ -1414,6 +1665,46 @@ InternalNode<ChildT, Log2Dim>::fill(const CoordBBox& bbox, const ValueType& valu
 ////////////////////////////////////////
 
 
+template<typename ChildT, Index Log2Dim>
+template<typename DenseT>
+inline void
+InternalNode<ChildT, Log2Dim>::copyToDense(const CoordBBox& bbox, DenseT& dense) const
+{
+    const size_t xStride = dense.xStride(), yStride = dense.yStride();// zStride=1
+    const Coord& min = dense.bbox().min();
+    for (Coord xyz = bbox.min(), max; xyz[0] <= bbox.max()[0]; xyz[0] = max[0] + 1) {
+        for (xyz[1] = bbox.min()[1]; xyz[1] <= bbox.max()[1]; xyz[1] = max[1] + 1) {
+            for (xyz[2] = bbox.min()[2]; xyz[2] <= bbox.max()[2]; xyz[2] = max[2] + 1) {
+                const Index n = this->coord2offset(xyz);
+                // Get max coordinates of the child node that contains voxel xyz.
+                max = this->offset2globalCoord(n).offsetBy(ChildT::DIM-1);
+
+                // Get the bbox of the interection of bbox and the child node
+                CoordBBox sub(xyz, Coord::minComponent(bbox.max(), max));
+
+                if (this->isChildMaskOn(n)) {//is a child
+                    mNodes[n].getChild()->copyToDense(sub, dense);
+                } else {//a tile value
+                    const ValueType value = mNodes[n].getValue();
+                    sub.translate(-min);
+                    ValueType* a0 = dense.data() + sub.min()[2];
+                    for (Int32 x=sub.min()[0], ex=sub.max()[0]+1; x<ex; ++x) {
+                        ValueType* a1 = a0 + x*xStride;
+                        for (Int32 y=sub.min()[1], ey=sub.max()[1]+1; y<ey; ++y) {
+                            ValueType* a2 = a1 + y*yStride;
+                            for (Int32 z=sub.min()[2], ez=sub.max()[2]+1; z<ez; ++z) *a2++ = value;
+                        }
+                    }
+                }
+            }
+        }
+    }
+}
+
+
+////////////////////////////////////////
+
+
 template<typename ChildT, Index Log2Dim>
 inline void
 InternalNode<ChildT, Log2Dim>::writeTopology(std::ostream& os, bool toHalf) const
@@ -1512,6 +1803,8 @@ InternalNode<ChildT, Log2Dim>::getLastValue() const
 
 
 ////////////////////////////////////////
+
+
 template<typename ChildT, Index Log2Dim>
 inline void
 InternalNode<ChildT, Log2Dim>::signedFloodFill(const ValueType& background)
@@ -1519,6 +1812,7 @@ InternalNode<ChildT, Log2Dim>::signedFloodFill(const ValueType& background)
     this->signedFloodFill(background, negative(background));
 }
 
+
 template<typename ChildT, Index Log2Dim>
 inline void
 InternalNode<ChildT, Log2Dim>::signedFloodFill(const ValueType& outsideValue,
@@ -1575,6 +1869,7 @@ InternalNode<ChildT, Log2Dim>::negate()
 
 }
 
+
 template<typename ChildT, Index Log2Dim>
 inline void
 InternalNode<ChildT, Log2Dim>::voxelizeActiveTiles()
@@ -1589,6 +1884,7 @@ InternalNode<ChildT, Log2Dim>::voxelizeActiveTiles()
     for (ChildOnIter iter = this->beginChildOn(); iter; ++iter) iter->voxelizeActiveTiles();
 }
 
+
 template<typename ChildT, Index Log2Dim>
 inline void
 InternalNode<ChildT, Log2Dim>::merge(InternalNode& other,
@@ -1653,6 +1949,7 @@ InternalNode<ChildT, Log2Dim>::topologyUnion(const InternalNode<OtherChildT, Log
     }
 }
 
+
 ////////////////////////////////////////
 
 
