@@ -82,7 +82,7 @@ public:
     struct ValueConverter {
         typedef LeafNode<OtherValueType, Log2Dim> Type;
     };
-    
+
     /// @brief Stores the actual values in the LeafNode. Its dimension
     /// it fixed to 2^(3*Log2Dim)
     class Buffer
@@ -100,8 +100,8 @@ public:
         void fill(const ValueType& val)
         {
             ValueType* target = mData;
-            Index size = SIZE;
-            while (size--) *target++ = val;
+            Index n = SIZE;
+            while (n--) *target++ = val;
         }
         /// Return a const reference to the i'th element of the Buffer
         const ValueType& getValue(Index i) const { assert(i < SIZE); return mData[i]; }
@@ -114,8 +114,8 @@ public:
         {
             ValueType* target = mData;
             const ValueType* source = other.mData;
-            Index size = SIZE;
-            while (size--) *target++ = *source++;
+            Index n = SIZE;
+            while (n--) *target++ = *source++;
             return *this;
         }
         /// Return true if the values in the other buffer exactly
@@ -124,14 +124,14 @@ public:
         {
             const ValueType* target = mData;
             const ValueType* source = other.mData;
-            Index size = SIZE;
-            while (size--) if(!math::isExactlyEqual(*target++, *source++)) return false;
-            return true;
+            Index n = SIZE;
+            while (n && math::isExactlyEqual(*target++, *source++)) --n;
+            return n == 0;
         }
         /// Return true if any of the values in the other buffer do
         /// not exactly equate the values in this buffer
         bool operator!=(const Buffer& other) const { return !(other == *this); }
-        /// Replace the values in this Buffer with the values in the other Buffer 
+        /// Replace the values in this Buffer with the values in the other Buffer
         void swap(Buffer& other)
         {
             ValueType* tmp = mData;
@@ -226,7 +226,10 @@ public:
     /// spanned by this leaf node.
     CoordBBox getNodeBoundingBox() const { return CoordBBox::createCube(mOrigin, DIM); }
 
+    /// Set the grid index coordinates of this node's local origin.
+    void setOrigin(const Coord& origin)  { mOrigin = origin; }
     /// Get the grid index coordinates of this node's local origin.
+    const Coord& origin() const { return mOrigin; }
     const Coord& getOrigin() const { return mOrigin; }
     void getOrigin(Coord& origin) const { origin = mOrigin; }
     void getOrigin(Int32& x, Int32& y, Int32& z) const { mOrigin.asXYZ(x, y, z); }
@@ -537,6 +540,40 @@ public:
     /// @brief Sets all values to the specified value and state
     void fill(const ValueType& value, bool active);
 
+    /// @brief Copy into a dense grid the values of the voxels that lie within
+    /// a given bounding box.
+    ///
+    /// @param bbox   inclusive bounding box of the voxels to be copied into the dense grid
+    /// @param dense  dense grid with a stride in @e z of one (see tools::Dense
+    ///               in tools/Dense.h for the required API)
+    ///
+    /// @note @a bbox is assumed to be identical to or contained in the coordinate domains
+    /// of both the dense grid and this node, i.e., no bounds checking is performed.
+    /// @note Consider using tools::CopyToDense in tools/Dense.h
+    /// instead of calling this method directly.
+    template<typename DenseT>
+    void copyToDense(const CoordBBox& bbox, DenseT& dense) const;
+
+    /// @brief Copy from a dense grid into this node the values of the voxels
+    /// that lie within a given bounding box.
+    /// @details Only values that are different (by more than the given tolerance)
+    /// from the background value will be active.  Other values are inactive
+    /// and truncated to the background value.
+    ///
+    /// @param bbox        inclusive bounding box of the voxels to be copied into this node
+    /// @param dense       dense grid with a stride in @e z of one (see tools::Dense
+    ///                    in tools/Dense.h for the required API)
+    /// @param background  background value of the tree that this node belongs to
+    /// @param tolerance   tolerance within which a value equals the background value
+    ///
+    /// @note @a bbox is assumed to be identical to or contained in the coordinate domains
+    /// of both the dense grid and this node, i.e., no bounds checking is performed.
+    /// @note Consider using tools::CopyFromDense in tools/Dense.h
+    /// instead of calling this method directly.
+    template<typename DenseT>
+    void copyFromDense(const CoordBBox& bbox, const DenseT& dense,
+                       const ValueType& background, const ValueType& tolerance);
+
     /// @brief Return the value of the voxel at the given coordinates.
     /// @note Used internally by ValueAccessor (note last argument is a dummy).
     template<typename AccessorT>
@@ -630,12 +667,12 @@ public:
     /// @brief Replace inactive occurrences of @a oldBackground with @a newBackground,
     /// and inactive occurrences of @a -oldBackground with @a -newBackground.
     void resetBackground(const ValueType& oldBackground, const ValueType& newBackground);
-    
+
     /// @brief Overwrites the inactive voxels with a new value whos
     /// magnitude is equal to the specified background value and sign
     /// is consistant with the lexicographically closest active voxel.
     /// The net effect is a propagation of signs from the active
-    /// voxels to the inactive voxels. 
+    /// voxels to the inactive voxels.
     ///
     /// @note This method is primarily useful for propagating the sign
     /// from the (active) voxels in a narrow-band level set to the
@@ -660,15 +697,20 @@ public:
     void voxelizeActiveTiles() {};
 
     /// @brief Union this node's set of active values with the active values
-    /// of the other node, whose @c ValueType may be different.
+    /// of the other node, whose @c ValueType may be different. So a
+    /// resulting voxel will be active if either of the original voxels
+    /// were active.
     ///
     /// @note This operation modifies only active states, not values.
     template<typename OtherType>
     void topologyUnion(const LeafNode<OtherType, Log2Dim>& other);
 
     /// @brief Intersect this node's set of active values with the active values
-    /// of the other node, whose @c ValueType may be different.
-    /// @details The last dummy arguemnt is required to match the signature
+    /// of the other node, whose @c ValueType may be different. So a
+    /// resulting voxel will be active only if both of the original voxels
+    /// were active.
+    ///
+    /// @details The last dummy argument is required to match the signature
     /// for InternalNode::topologyIntersection.
     ///
     /// @note This operation modifies only active states, not
@@ -677,6 +719,20 @@ public:
     template<typename OtherType>
     void topologyIntersection(const LeafNode<OtherType, Log2Dim>& other, const ValueType&);
 
+    /// @brief Difference this node's set of active values with the active values
+    /// of the other node, whose @c ValueType may be different. So a
+    /// resulting voxel will be active only if both of the original
+    /// voxel is active in this LeafNode and inactive in the other LeafNode.
+    ///
+    /// @details The last dummy argument is required to match the signature
+    /// for InternalNode::topologyDifference.
+    ///
+    /// @note This operation modifies only active states, not
+    /// values. Also note that this operation can result in all voxels
+    /// being inactive so consider subsequnetly calling prune.
+    template<typename OtherType>
+    void topologyDifference(const LeafNode<OtherType, Log2Dim>& other, const ValueType&);
+
     template<typename CombineOp>
     void combine(const LeafNode& other, CombineOp& op);
     template<typename CombineOp>
@@ -713,9 +769,17 @@ public:
     template<typename PruneOp> void pruneOp(PruneOp&) {}
     void prune(const ValueType& /*tolerance*/ = zeroVal<ValueType>()) {}
     void pruneInactive(const ValueType&) {}
+    void addLeaf(LeafNode*) {}
+    template<typename AccessorT>
+    void addLeafAndCache(LeafNode*, AccessorT&) {}
+    template<typename NodeT>
+    NodeT* stealNode(const Coord&, const ValueType&, bool) { return NULL; }
+    void addTile(Index, const Coord&, const ValueType&, bool) {}
+    template<typename AccessorT>
+    void addTileAndCache(Index, const Coord&, const ValueType&, bool, AccessorT&) {}
     //@}
     //@{
-    /// @brief return a pointer to itself
+    /// @brief Return a pointer to this node.
     LeafNode* touchLeaf(const Coord&) { return this; }
     template<typename AccessorT>
     LeafNode* touchLeafAndCache(const Coord&, AccessorT&) { return this; }
@@ -724,10 +788,13 @@ public:
     LeafNode* probeLeafAndCache(const Coord&, AccessorT&) { return this; }
     //@}
     //@{
-    /// @brief return a const pointer to itself
+    /// @brief Return a @const pointer to this node.
     const LeafNode* probeConstLeaf(const Coord&) const { return this; }
     template<typename AccessorT>
     const LeafNode* probeConstLeafAndCache(const Coord&, AccessorT&) const { return this; }
+    template<typename AccessorT>
+    const LeafNode* probeLeafAndCache(const Coord&, AccessorT&) const { return this; }
+    const LeafNode* probeLeaf(const Coord&) const { return this; }
     //@}
 
     /// Return @c true if all of this node's values have the same active state
@@ -1028,6 +1095,57 @@ LeafNode<T, Log2Dim>::fill(const ValueType& value, bool active)
     mValueMask.set(active);
 }
 
+////////////////////////////////////////
+
+template<typename T, Index Log2Dim>
+template<typename DenseT>
+inline void
+LeafNode<T, Log2Dim>::copyToDense(const CoordBBox& bbox, DenseT& dense) const
+{
+    const size_t xStride = dense.xStride(), yStride = dense.yStride();// zStride=1
+    const Coord& min = dense.bbox().min();
+    T* t0 = dense.data() + bbox.min()[2]-min[2];//target array
+    const T* s0 = &mBuffer[bbox.min()[2]&DIM-1u];//source array
+    for (Int32 x = bbox.min()[0], ex=bbox.max()[0]+1; x<ex; ++x) {
+        T*       t1 = t0 + xStride*(x-min[0]);
+        const T* s1 = s0 + ((x&DIM-1u)<<2*Log2Dim);
+        for (Int32 y = bbox.min()[1], ey=bbox.max()[1]+1; y<ey; ++y) {
+            T*       t2 = t1 + yStride*(y-min[1]);
+            const T* s2 = s1 + ((y&DIM-1u)<<Log2Dim);
+            for (Int32 z = bbox.min()[2], ez=bbox.max()[2]+1; z<ez ; ++z) *t2++ = *s2++;
+        }
+    }
+}
+
+template<typename T, Index Log2Dim>
+template<typename DenseT>
+inline void
+LeafNode<T, Log2Dim>::copyFromDense(const CoordBBox& bbox, const DenseT& dense,
+                                    const ValueType& background, const ValueType& tolerance)
+{
+    const size_t xStride = dense.xStride(), yStride = dense.yStride();// zStride=1
+    const Coord& min = dense.bbox().min();
+
+    const ValueType* s0 = dense.data() + bbox.min()[2]-min[2];//source
+    const Int32      n0 = bbox.min()[2]&DIM-1u;
+    for (Int32 x = bbox.min()[0], ex=bbox.max()[0]+1; x<ex; ++x) {
+        const ValueType* s1 = s0 + xStride*(x-min[0]);
+        const Int32      n1 = n0 + ((x&DIM-1u)<<2*LOG2DIM);
+        for (Int32 y = bbox.min()[1], ey=bbox.max()[1]+1; y<ey; ++y) {
+            const ValueType* s2 = s1 + yStride*(y-min[1]);
+            Int32            n2 = n1 + ((y&DIM-1u)<<LOG2DIM) ;
+            for (Int32 z = bbox.min()[2], ez=bbox.max()[2]+1; z<ez ; ++z, ++n2, ++s2) {
+                if (math::isApproxEqual(background, *s2, tolerance)) {
+                    mValueMask.setOff(n2);
+                    mBuffer[n2] = background;
+                } else {
+                    mValueMask.setOn(n2);
+                    mBuffer[n2] = *s2;
+                }
+            }
+        }
+    }
+}
 
 ////////////////////////////////////////
 
@@ -1254,6 +1372,15 @@ LeafNode<T, Log2Dim>::topologyIntersection(const LeafNode<OtherType, Log2Dim>& o
     mValueMask &= other.getValueMask();
 }
 
+template<typename T, Index Log2Dim>
+template<typename OtherType>
+inline void
+LeafNode<T, Log2Dim>::topologyDifference(const LeafNode<OtherType, Log2Dim>& other,
+                                         const ValueType&)
+{
+    mValueMask &= !other.getValueMask();
+}
+
 template<typename T, Index Log2Dim>
 inline void
 LeafNode<T, Log2Dim>::negate()
