@@ -177,10 +177,15 @@ public:
     /// spanned by this leaf node.
     CoordBBox getNodeBoundingBox() const { return CoordBBox::createCube(mOrigin, DIM); }
 
+    /// Set the grid index coordinates of this node's local origin.
+    void setOrigin(const Coord& origin)  { mOrigin = origin; }
+    //@{
     /// Get the grid index coordinates of this node's local origin.
+    const Coord& origin() const { return mOrigin; }
     const Coord& getOrigin() const { return mOrigin; }
     void getOrigin(Coord& origin) const { origin = mOrigin; }
     void getOrigin(Int32& x, Int32& y, Int32& z) const { mOrigin.asXYZ(x, y, z); }
+    //@}
 
     /// Return the linear table offset of the given coordinates.
     static Index coord2offset(const Coord& xyz);
@@ -306,6 +311,39 @@ public:
     /// @brief Sets all values to the specified value and state
     void fill(const bool& value, bool active);
 
+    /// @brief Copy into a dense grid the values of the voxels that lie within
+    /// a given bounding box.
+    ///
+    /// @param bbox   inclusive bounding box of the voxels to be copied into the dense grid
+    /// @param dense  dense grid with a stride in @e z of one (see tools::Dense
+    ///               in tools/Dense.h for the required API)
+    ///
+    /// @note @a bbox is assumed to be identical to or contained in the coordinate domains
+    /// of both the dense grid and this node, i.e., no bounds checking is performed.
+    /// @note Consider using tools::CopyToDense in tools/Dense.h
+    /// instead of calling this method directly.
+    template<typename DenseT>
+    void copyToDense(const CoordBBox& bbox, DenseT& dense) const;
+
+    /// @brief Copy from a dense grid into this node the values of the voxels
+    /// that lie within a given bounding box.
+    /// @details Only values that are different (by more than the given tolerance)
+    /// from the background value will be active.  Other values are inactive
+    /// and truncated to the background value.
+    ///
+    /// @param bbox        inclusive bounding box of the voxels to be copied into this node
+    /// @param dense       dense grid with a stride in @e z of one (see tools::Dense
+    ///                    in tools/Dense.h for the required API)
+    /// @param background  background value of the tree that this node belongs to
+    /// @param tolerance   tolerance within which a value equals the background value
+    ///
+    /// @note @a bbox is assumed to be identical to or contained in the coordinate domains
+    /// of both the dense grid and this node, i.e., no bounds checking is performed.
+    /// @note Consider using tools::CopyFromDense in tools/Dense.h
+    /// instead of calling this method directly.
+    template<typename DenseT>
+    void copyFromDense(const CoordBBox& bbox, const DenseT& dense, bool background, bool tolerance);
+
     /// @brief Return the value of the voxel at the given coordinates.
     /// @note Used internally by ValueAccessor (note last argument is a dummy).
     template<typename AccessorT>
@@ -383,15 +421,20 @@ public:
     void voxelizeActiveTiles() {};
 
     /// @brief Union this node's set of active values with the active values
-    /// of the other node, whose @c ValueType may be different.
+    /// of the other node, whose @c ValueType may be different. So a
+    /// resulting voxel will be active if either of the original voxels
+    /// were active.
     ///
     /// @note This operation modifies only active states, not values.
     template<typename OtherType>
     void topologyUnion(const LeafNode<OtherType, Log2Dim>& other);
 
     /// @brief Intersect this node's set of active values with the active values
-    /// of the other node, whose @c ValueType may be different.
-    /// @details The last dummy arguemnt is required to match the signature
+    /// of the other node, whose @c ValueType may be different. So a
+    /// resulting voxel will be active only if both of the original voxels
+    /// were active.
+    ///
+    /// @details The last dummy argument is required to match the signature
     /// for InternalNode::topologyIntersection.
     ///
     /// @note This operation modifies only active states, not
@@ -400,6 +443,20 @@ public:
     template<typename OtherType>
     void topologyIntersection(const LeafNode<OtherType, Log2Dim>& other, const bool&);
 
+    /// @brief Difference this node's set of active values with the active values
+    /// of the other node, whose @c ValueType may be different. So a
+    /// resulting voxel will be active only if both of the original
+    /// voxel is active in this LeafNode and inactive in the other LeafNode.
+    ///
+    /// @details The last dummy argument is required to match the signature
+    /// for InternalNode::topologyDifference.
+    ///
+    /// @note This operation modifies only active states, not
+    /// values. Also note that this operation can result in all voxels
+    /// being inactive so consider subsequnetly calling prune.
+    template<typename OtherType>
+    void topologyDifference(const LeafNode<OtherType, Log2Dim>& other, const bool&);
+
     template<typename CombineOp>
     void combine(const LeafNode& other, CombineOp& op);
     template<typename CombineOp>
@@ -438,6 +495,13 @@ public:
     template<typename PruneOp> void pruneOp(PruneOp&) {}
     void prune(const ValueType& /*tolerance*/ = zeroVal<ValueType>()) {}
     void pruneInactive(const ValueType&) {}
+    void addLeaf(LeafNode*) {}
+    template<typename AccessorT>
+    void addLeafAndCache(LeafNode*, AccessorT&) {}
+    LeafNode* stealLeaf(const Coord&, bool, bool) { return this; }
+    void addTile(Index, const Coord&, bool, bool) {}
+    template<typename AccessorT>
+    void addTileAndCache(Index, const Coord&, const ValueType&, bool, AccessorT&) {}
     //@}
 
     //@{
@@ -451,11 +515,14 @@ public:
     //@}
     //@{
     /// @brief return a const pointer to itself
+    const LeafNode* probeLeaf(const Coord&) const { return this; }
+    template<typename AccessorT>
+    const LeafNode* probeLeafAndCache(const Coord&, AccessorT&) const { return this; }
     const LeafNode* probeConstLeaf(const Coord&) const { return this; }
     template<typename AccessorT>
     const LeafNode* probeConstLeafAndCache(const Coord&, AccessorT&) const { return this; }
     //@}
-
+    
     void merge(const LeafNode& other, bool, bool) { this->merge(other); }
 
     //
@@ -1070,6 +1137,15 @@ LeafNode<bool, Log2Dim>::topologyIntersection(const LeafNode<OtherType, Log2Dim>
     mValueMask &= other.getValueMask();
 }
 
+template<Index Log2Dim>
+template<typename OtherType>
+inline void
+LeafNode<bool, Log2Dim>::topologyDifference(const LeafNode<OtherType, Log2Dim>& other,
+                                            const bool&)
+{
+    mValueMask &= !other.getValueMask();
+}
+
 template<Index Log2Dim>
 inline void
 LeafNode<bool, Log2Dim>::fill(const CoordBBox& bbox, bool value, bool active)
@@ -1102,6 +1178,64 @@ LeafNode<bool, Log2Dim>::fill(const bool& value, bool active)
     mValueMask.set(active);
 }
 
+
+////////////////////////////////////////
+
+
+template<Index Log2Dim>
+template<typename DenseT>
+inline void
+LeafNode<bool, Log2Dim>::copyToDense(const CoordBBox& bbox, DenseT& dense) const
+{
+    const size_t xStride = dense.xStride(), yStride = dense.yStride();// zStride=1
+    const Coord& min = dense.bbox().min();
+    bool*       t0 = dense.data() + bbox.min()[2]-min[2];//target array
+    const Int32 n0 = bbox.min()[2]&DIM-1u;
+    for (Int32 x = bbox.min()[0], ex=bbox.max()[0]+1; x<ex; ++x) {
+        bool*       t1 = t0 + xStride*(x-min[0]);
+        const Int32 n1 = n0 + ((x&DIM-1u)<<2*LOG2DIM);
+        for (Int32 y = bbox.min()[1], ey=bbox.max()[1]+1; y<ey; ++y) {
+            bool* t2 = t1 + yStride*(y-min[1]);
+            Int32 n2 = n1 + ((y&DIM-1u)<<LOG2DIM) ;
+            for (Int32 z = bbox.min()[2], ez=bbox.max()[2]+1; z<ez ; ++z) {
+                *t2++ = mBuffer.mData.isOn(n2++);
+            }
+        }
+    }
+}
+
+
+template<Index Log2Dim>
+template<typename DenseT>
+inline void
+LeafNode<bool, Log2Dim>::copyFromDense(const CoordBBox& bbox, const DenseT& dense,
+                                       bool background, bool tolerance)
+{
+    const size_t xStride = dense.xStride(), yStride = dense.yStride();// zStride=1
+    const Coord& min = dense.bbox().min();
+    const bool* s0 = dense.data() + bbox.min()[2]-min[2];//source
+    const Int32      n0 = bbox.min()[2]&DIM-1u;
+    for (Int32 x = bbox.min()[0], ex=bbox.max()[0]+1; x<ex; ++x) {
+        const bool* s1 = s0 + xStride*(x-min[0]);
+        const Int32      n1 = n0 + ((x&DIM-1u)<<2*LOG2DIM);
+        for (Int32 y = bbox.min()[1], ey=bbox.max()[1]+1; y<ey; ++y) {
+            const bool* s2 = s1 + yStride*(y-min[1]);
+            Int32            n2 = n1 + ((y&DIM-1u)<<LOG2DIM) ;
+            for (Int32 z = bbox.min()[2], ez=bbox.max()[2]+1; z<ez ; ++z, ++n2, ++s2) {
+                // Note: if tolerance is true (i.e., 1), then all boolean values compare equal.
+                if (tolerance || background == *s2) {
+                    mValueMask.setOff(n2);
+                    mBuffer.mData.set(n2, background);
+                } else {
+                    mValueMask.setOn(n2);
+                    mBuffer.mData.set(n2, *s2);
+                }
+            }
+        }
+    }
+}
+
+
 ////////////////////////////////////////
 
 
