@@ -440,7 +440,7 @@ public:
     /// Return the current index range.  Both min and max are inclusive.
     void getIndexRange(CoordBBox& bbox) const;
 
-     /// @brief Return @c true if the given tree has the same node and active value
+    /// @brief Return @c true if the given tree has the same node and active value
     /// topology as this tree (but possibly a different @c ValueType).
     template<typename OtherChildType>
     bool hasSameTopology(const RootNode<OtherChildType>& other) const;
@@ -490,6 +490,14 @@ public:
     ///                       otherwise mark them as inactive
     void fill(const CoordBBox& bbox, const ValueType& value, bool active = true);
 
+    /// @brief Copy into a dense grid the values of all voxels, both active and inactive,
+    /// that intersect a given bounding box.
+    ///
+    /// @param bbox   inclusive bounding box of the voxels to be copied into the dense grid
+    /// @param dense  dense grid with a stride in @e z of one (see tools::Dense
+    ///               in tools/Dense.h for the required API)
+    template<typename DenseT>
+    void copyToDense(const CoordBBox& bbox, DenseT& dense) const;
 
     //
     // I/O
@@ -585,6 +593,38 @@ public:
     /// background tiles any nodes whose values are all inactive.
     void pruneInactive();
 
+    void pruneTiles(const ValueType&);
+
+    /// @brief Add the specified leaf to this node, possibly creating a child branch
+    /// in the process.  If the leaf node already exists, replace it.
+    void addLeaf(LeafNodeType* leaf);
+
+    /// @brief Same as addLeaf except, if necessary, it update the accessor with pointers
+    /// to the nodes along the path from the root node to the node containing the coordinate.
+    template<typename AccessorT>
+    void addLeafAndCache(LeafNodeType* leaf, AccessorT&);
+
+    /// @brief Return a pointer to the node of type @c NodeT that contains voxel (x, y, z)
+    /// and replace it with a tile of the specified value and state.
+    /// If no such node exists, leave the tree unchanged and return @c NULL.
+    ///
+    /// @note The caller takes ownership of the node and is responsible for deleting it.
+    ///
+    /// @warning Since this method potentially removes nodes and branches of the tree,
+    /// it is important to clear the caches of all ValueAccessors associated with this tree.
+    template<typename NodeT>
+    NodeT* stealNode(const Coord& xyz, const ValueType& value, bool state);
+
+    /// @brief Add a tile at the specified tree level that contains voxel (x, y, z),
+    /// possibly creating a parent branch or deleting a child branch in the process.
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool state);
+
+    /// @brief Same as addTile() except, if necessary, update the accessor with pointers
+    /// to the nodes along the path from the root node to the node containing (x, y, z).
+    template<typename AccessorT>
+    void addTileAndCache(Index level, const Coord& xyz, const ValueType& value,
+        bool state, AccessorT&);
+
     /// @brief Return the leaf node that contains voxel (x, y, z).
     /// If no such node exists, create one, but preserve the values and
     /// active states of all voxels.
@@ -593,26 +633,34 @@ public:
     /// over which to safely perform multithreaded processing.
     LeafNodeType* touchLeaf(const Coord& xyz);
 
-    /// @brief Return a pointer to the leaf node that contains voxel (x, y, z).
-    /// If no such node exists, return NULL.
-    LeafNodeType* probeLeaf(const Coord& xyz);
-    /// @brief Return a const pointer to the leaf node that contains voxel (x, y, z).
-    /// If no such node exists, return NULL.
-    const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
-
     /// @brief Same as touchLeaf except, if necessary, it update the accessor with pointers
     /// to the nodes along the path from the root node to the node containing the coordinate.
     template<typename AccessorT>
-    LeafNodeType* touchLeafAndCache(const Coord& xyz, AccessorT&);
+    LeafNodeType* touchLeafAndCache(const Coord& xyz, AccessorT& acc);
+
+    /// @brief Return a pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, return NULL.
+    LeafNodeType* probeLeaf(const Coord& xyz);
+
+    /// @brief Return a const pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, return NULL.
+    const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
+    const LeafNodeType* probeLeaf(const Coord& xyz) const { return this->probeConstLeaf(xyz); }
 
     /// @brief Same as probeLeaf except, if necessary, it update the accessor with pointers
     /// to the nodes along the path from the root node to the node containing the coordinate.
     template<typename AccessorT>
-    LeafNodeType* probeLeafAndCache(const Coord& xyz, AccessorT&);
+    LeafNodeType* probeLeafAndCache(const Coord& xyz, AccessorT& acc);
+
     /// @brief Same as probeConstLeaf except, if necessary, it update the accessor with pointers
     /// to the nodes along the path from the root node to the node containing the coordinate.
     template<typename AccessorT>
-    const LeafNodeType* probeConstLeafAndCache(const Coord& xyz, AccessorT&) const;
+    const LeafNodeType* probeConstLeafAndCache(const Coord& xyz, AccessorT& acc) const;
+    template<typename AccessorT>
+    const LeafNodeType* probeLeafAndCache(const Coord& xyz, AccessorT& acc) const
+    {
+        return this->probeConstLeafAndCache(xyz, acc);
+    }
 
     /// @brief Set the values of all inactive voxels and tiles of a narrow-band
     /// level set from the signs of the active voxels, setting outside values to
@@ -622,9 +670,9 @@ public:
     void signedFloodFill();
 
     /// @brief Set the values of all inactive voxels and tiles of a narrow-band
-    /// level set from the signs of the active voxels, setting outside values to
-    /// "outside" and inside values to "inside. The background value of the grid
-    /// will be set to the "outside" value.
+    /// level set from the signs of the active voxels, setting exterior values to
+    /// @a outside and interior values to @a inside.  Set the background value
+    /// of this tree to @a outside.
     ///
     /// @note This method should only be used on closed, narrow-band level sets!
     void signedFloodFill(const ValueType& outside, const ValueType& inside);
@@ -721,6 +769,9 @@ private:
     /// @return an iterator pointing to the matching mTable entry.
     MapIter findOrAddCoord(const Coord& xyz);
 
+    template<typename NodeT>
+    NodeT* doStealNode(const Coord&, const ValueType&, bool state);
+
     /// @throw TypeError if the other node's dimensions don't match this node's.
     template<typename OtherChildType>
     static void enforceSameConfiguration(const RootNode<OtherChildType>& other);
@@ -1662,6 +1713,43 @@ RootNode<ChildT>::fill(const CoordBBox& bbox, const ValueType& value, bool activ
     }
 }
 
+template<typename ChildT>
+template<typename DenseT>
+inline void
+RootNode<ChildT>::copyToDense(const CoordBBox& bbox, DenseT& dense) const
+{
+    const size_t xStride = dense.xStride(), yStride = dense.yStride();// zStride=1
+    const Coord& min = dense.bbox().min();
+    CoordBBox nodeBBox;
+    for (Coord xyz = bbox.min(); xyz[0] <= bbox.max()[0]; xyz[0] = nodeBBox.max()[0] + 1) {
+        for (xyz[1] = bbox.min()[1]; xyz[1] <= bbox.max()[1]; xyz[1] = nodeBBox.max()[1] + 1) {
+            for (xyz[2] = bbox.min()[2]; xyz[2] <= bbox.max()[2]; xyz[2] = nodeBBox.max()[2] + 1) {
+
+                // Get the coordinate bbox of the child node that contains voxel xyz.
+                nodeBBox = CoordBBox::createCube(coordToKey(xyz), ChildT::DIM);
+
+                // Get the coordinate bbox of the interection of inBBox and nodeBBox
+                CoordBBox sub(xyz, Coord::minComponent(bbox.max(), nodeBBox.max()));
+
+                MapCIter iter = this->findKey(nodeBBox.min());
+                if (iter != mTable.end() && isChild(iter)) {//is a child
+                    getChild(iter).copyToDense(sub, dense);
+                } else {//is background or a tile value
+                    const ValueType value = iter==mTable.end() ? mBackground : getTile(iter).value;
+                    sub.translate(-min);
+                    ValueType* a0 = dense.data() + sub.min()[2];
+                    for (Int32 x=sub.min()[0], ex=sub.max()[0]+1; x<ex; ++x) {
+                        ValueType* a1 = a0 + x*xStride;
+                        for (Int32 y=sub.min()[1], ey=sub.max()[1]+1; y<ey; ++y) {
+                            ValueType* a2 = a1 + y*yStride;
+                            for (Int32 z=sub.min()[2], ez=sub.max()[2]+1; z<ez; ++z) *a2++ = value;
+                        }
+                    }
+                }
+            }
+        }
+    }
+}
 
 ////////////////////////////////////////
 
@@ -1870,6 +1958,196 @@ RootNode<ChildT>::pruneInactive()
 }
 
 
+template<typename ChildT>
+inline void
+RootNode<ChildT>::pruneTiles(const ValueType& tolerance)
+{
+    TolerancePrune<ValueType, /*Terminate at level=*/1> op(tolerance);
+    this->pruneOp(op);
+}
+
+
+////////////////////////////////////////
+
+
+// Helper method that implements stealNode()
+template<typename ChildT>
+template<typename NodeT>
+inline NodeT*
+RootNode<ChildT>::doStealNode(const Coord& xyz, const ValueType& value, bool state)
+{
+    MapIter iter = this->findCoord(xyz);
+    if (iter == mTable.end() || isTile(iter)) return NULL;
+    return (boost::is_same<NodeT, ChildT>::value)
+        ? reinterpret_cast<NodeT*>(&stealChild(iter, Tile(value, state)))
+        : getChild(iter).template stealNode<NodeT>(xyz, value, state);
+}
+
+
+template<typename ChildT>
+template<typename NodeT>
+inline NodeT*
+RootNode<ChildT>::stealNode(const Coord& xyz, const ValueType& value, bool state)
+{
+    // The following conditional is resolved at compile time, and the ternary operator
+    // and helper method are used to avoid "unreachable code" warnings (with
+    // "if (<cond>) { <A> } else { <B> }", either <A> or <B> is unreachable if <cond>
+    // is a compile-time constant expression).  Partial specialization on NodeT would be
+    // impractical because a method template can't be specialized without also
+    // specializing its class template.
+    return (NodeT::LEVEL > ChildT::LEVEL ||
+        (NodeT::LEVEL == ChildT::LEVEL && !(boost::is_same<NodeT, ChildT>::value)))
+        ? static_cast<NodeT*>(NULL)
+        : this->doStealNode<NodeT>(xyz, value, state);
+}
+
+
+////////////////////////////////////////
+
+
+template<typename ChildT>
+inline void
+RootNode<ChildT>::addLeaf(LeafNodeType* leaf)
+{
+    if (leaf == NULL) return;
+    ChildT* child = NULL;
+    const Coord& xyz = leaf->origin();
+    MapIter iter = this->findCoord(xyz);
+    if (iter == mTable.end()) {
+        if (ChildT::LEVEL>0) {
+            child = new ChildT(xyz, mBackground, false);
+        } else {
+            child = reinterpret_cast<ChildT*>(leaf);
+        }
+        mTable[this->coordToKey(xyz)] = NodeStruct(*child);
+    } else if (isChild(iter)) {
+        if (ChildT::LEVEL>0) {
+            child = &getChild(iter);
+        } else {
+            child = reinterpret_cast<ChildT*>(leaf);
+            setChild(iter, *child);//this also deletes the existing child node
+        }
+    } else {//tile
+        if (ChildT::LEVEL>0) {
+            child = new ChildT(xyz, getTile(iter).value, isTileOn(iter));
+        } else {
+            child = reinterpret_cast<ChildT*>(leaf);
+        }
+        setChild(iter, *child);
+    }
+    child->addLeaf(leaf);
+}
+
+
+template<typename ChildT>
+template<typename AccessorT>
+inline void
+RootNode<ChildT>::addLeafAndCache(LeafNodeType* leaf, AccessorT& acc)
+{
+    if (leaf == NULL) return;
+    ChildT* child = NULL;
+    const Coord& xyz = leaf->origin();
+    MapIter iter = this->findCoord(xyz);
+    if (iter == mTable.end()) {
+        if (ChildT::LEVEL>0) {
+            child = new ChildT(xyz, mBackground, false);
+        } else {
+            child = reinterpret_cast<ChildT*>(leaf);
+        }
+        mTable[this->coordToKey(xyz)] = NodeStruct(*child);
+    } else if (isChild(iter)) {
+        if (ChildT::LEVEL>0) {
+            child = &getChild(iter);
+        } else {
+            child = reinterpret_cast<ChildT*>(leaf);
+            setChild(iter, *child);//this also deletes the existing child node
+        }
+    } else {//tile
+        if (ChildT::LEVEL>0) {
+            child = new ChildT(xyz, getTile(iter).value, isTileOn(iter));
+        } else {
+            child = reinterpret_cast<ChildT*>(leaf);
+        }
+        setChild(iter, *child);
+    }
+    acc.insert(xyz, child);
+    child->addLeafAndCache(leaf, acc);
+}
+
+
+template<typename ChildT>
+inline void
+RootNode<ChildT>::addTile(Index level, const Coord& xyz,
+                          const ValueType& value, bool state)
+{
+    if (LEVEL >= level && level > 0) {
+        MapIter iter = this->findCoord(xyz);
+        if (iter == mTable.end()) {//background
+            if (LEVEL > level) {
+                ChildT* child = new ChildT(xyz, mBackground, false);
+                mTable[this->coordToKey(xyz)] = NodeStruct(*child);
+                child->addTile(level, xyz, value, state);
+            } else {
+                mTable[this->coordToKey(xyz)] = NodeStruct(Tile(value, state));
+            }
+        } else if (isChild(iter)) {//child
+            if (LEVEL > level) {
+                getChild(iter).addTile(level, xyz, value, state);
+            } else {
+                setTile(iter, Tile(value, state));//this also deletes the existing child node
+            }
+        } else {//tile
+            if (LEVEL > level) {
+                ChildT* child = new ChildT(xyz, getTile(iter).value, isTileOn(iter));
+                setChild(iter, *child);
+                child->addTile(level, xyz, value, state);
+            } else {
+                setTile(iter, Tile(value, state));
+            }
+        }
+    }
+}
+
+
+template<typename ChildT>
+template<typename AccessorT>
+inline void
+RootNode<ChildT>::addTileAndCache(Index level, const Coord& xyz, const ValueType& value,
+                                  bool state, AccessorT& acc)
+{
+    if (LEVEL >= level && level > 0) {
+        MapIter iter = this->findCoord(xyz);
+        if (iter == mTable.end()) {//background
+            if (LEVEL > level) {
+                ChildT* child = new ChildT(xyz, mBackground, false);
+                acc.insert(xyz, child);
+                mTable[this->coordToKey(xyz)] = NodeStruct(*child);
+                child->addTileAndCache(level, xyz, value, state, acc);
+            } else {
+                mTable[this->coordToKey(xyz)] = NodeStruct(Tile(value, state));
+            }
+        } else if (isChild(iter)) {//child
+            if (LEVEL > level) {
+                ChildT* child = &getChild(iter);
+                acc.insert(xyz, child);
+                child->addTileAndCache(level, xyz, value, state, acc);
+            } else {
+                setTile(iter, Tile(value, state));//this also deletes the existing child node
+            }
+        } else {//tile
+            if (LEVEL > level) {
+                ChildT* child = new ChildT(xyz, getTile(iter).value, isTileOn(iter));
+                acc.insert(xyz, child);
+                setChild(iter, *child);
+                child->addTileAndCache(level, xyz, value, state, acc);
+            } else {
+                setTile(iter, Tile(value, state));
+            }
+        }
+    }
+}
+
+
 ////////////////////////////////////////
 
 
@@ -1925,17 +2203,6 @@ RootNode<ChildT>::probeLeaf(const Coord& xyz)
     return getChild(iter).probeLeaf(xyz);
 }
 
-
-template<typename ChildT>
-inline const typename ChildT::LeafNodeType*
-RootNode<ChildT>::probeConstLeaf(const Coord& xyz) const
-{
-    MapCIter iter = this->findCoord(xyz);
-    if (iter == mTable.end() || isTile(iter)) return NULL;
-    return getChild(iter).probeConstLeaf(xyz);
-}
-
-
 template<typename ChildT>
 template<typename AccessorT>
 inline typename ChildT::LeafNodeType*
@@ -1948,6 +2215,14 @@ RootNode<ChildT>::probeLeafAndCache(const Coord& xyz, AccessorT& acc)
     return child->probeLeafAndCache(xyz, acc);
 }
 
+template<typename ChildT>
+inline const typename ChildT::LeafNodeType*
+RootNode<ChildT>::probeConstLeaf(const Coord& xyz) const
+{
+    MapCIter iter = this->findCoord(xyz);
+    if (iter == mTable.end() || isTile(iter)) return NULL;
+    return getChild(iter).probeConstLeaf(xyz);
+}
 
 template<typename ChildT>
 template<typename AccessorT>
