@@ -255,6 +255,9 @@ public:
 
     //@{
     /// Return this tree's root node.
+    RootNodeType& root() { return mRoot; }
+    const RootNodeType& root() const { return mRoot; }
+    // Deprecate the methods below
     RootNodeType& getRootNode() { return mRoot; }
     const RootNodeType& getRootNode() const { return mRoot; }
     //@}
@@ -420,6 +423,24 @@ public:
     /// values are limited to either an inside or outside value.
     void pruneLevelSet();
 
+    /// @brief Add the given leaf node to this tree, creating a new branch if necessary.
+    /// If a leaf node with the same origin already exists, replace it.
+    void addLeaf(LeafNodeType& leaf) { mRoot.addLeaf(&leaf); }
+
+    /// @brief Add a tile containing voxel (x, y, z) at the specified tree level,
+    /// creating a new branch if necessary.  Delete any existing lower-level nodes
+    /// that contain (x, y, z).
+    /// @note @c Level must be greater than zero (i.e., the level of leaf nodes)
+    /// and less than this tree's depth.
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool active);
+
+    /// @brief Return a pointer to the node of type @c NodeT that contains voxel (x, y, z)
+    /// and replace it with a tile of the specified value and state.
+    /// If no such node exists, leave the tree unchanged and return @c NULL.
+    /// @note The caller takes ownership of the node and is responsible for deleting it.
+    template<typename NodeT>
+    NodeT* stealNode(const Coord& xyz, const ValueType& value, bool active);
+
     /// @brief @return the leaf node that contains voxel (x, y, z) and
     /// if it doesn't exist, create it, but preserve the values and
     /// active states of all voxels.
@@ -431,10 +452,11 @@ public:
     /// @brief @return a pointer to the leaf node that contains
     /// voxel (x, y, z) and if it doesn't exist, return NULL.
     LeafNodeType* probeLeaf(const Coord& xyz);
+
     /// @brief @return a const pointer to the leaf node that contains
     /// voxel (x, y, z) and if it doesn't exist, return NULL.
     const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
-
+    const LeafNodeType* probeLeaf(const Coord& xyz) const { return this->probeConstLeaf(xyz); }
 
     //
     // Aux methods
@@ -483,35 +505,22 @@ public:
     void signedFloodFill() { mRoot.signedFloodFill(); }
 
     /// @brief Set the values of all inactive voxels and tiles of a narrow-band
-    /// level set from the signs of the active voxels, setting outside values to
-    /// "outside" and inside values to "inside. The background value of the grid
-    /// will be set to the "outside" value.
+    /// level set from the signs of the active voxels, setting exterior values to
+    /// @a outside and interior values to @a inside.  Set the background value
+    /// of this tree to @a outside.
     ///
     /// @note This method should only be used on closed, narrow-band level sets!
-    void signedFloodFill(const ValueType& outside, const ValueType& inside)
-    {
-        mRoot.signedFloodFill(outside, inside);
-    }
+    void signedFloodFill(const ValueType& outside, const ValueType& inside);
 
     /// Move child nodes from the other tree into this tree wherever those nodes
     /// correspond to constant-value tiles in this tree, and replace leaf-level
     /// inactive voxels in this tree with corresponding voxels in the other tree
     /// that are active.
     /// @note This operation always empties the other tree.
-    void merge(Tree& other)
-    {
-        this->clearAllAccessors();
-        other.clearAllAccessors();
-        mRoot.merge(other.mRoot);
-    }
+    void merge(Tree& other);
 
     /// Turns active tiles into dense voxels, i.e. active leaf nodes
-    void voxelizeActiveTiles()
-    {
-        this->clearAllAccessors();
-        mRoot.voxelizeActiveTiles();
-    }
-
+    void voxelizeActiveTiles();
 
     /// @brief Union this tree's set of active values with the active values
     /// of the other tree, whose @c ValueType may be different.
@@ -527,11 +536,7 @@ public:
     /// tiles or voxels that were inactive in this tree but active in the other tree
     /// are marked as active in this tree but left with their original values.
     template<typename OtherRootNodeType>
-    void topologyUnion(const Tree<OtherRootNodeType>& other)
-    {
-        this->clearAllAccessors();
-        mRoot.topologyUnion(other.getRootNode());
-    }
+    void topologyUnion(const Tree<OtherRootNodeType>& other);
 
     /*! For a given function @c f, use sparse traversal to compute <tt>f(this, other)</tt>
      *  over all corresponding pairs of values (tile or voxel) of this tree and the other tree
@@ -757,12 +762,7 @@ public:
      *  @endcode
      *  @see openvdb/unittest/TestTree.cc for another example.
      */
-
-    template<typename BBoxOp> void visitActiveBBox(BBoxOp& op) const
-    {
-        mRoot.visitActiveBBox(op);
-    }
-
+    template<typename BBoxOp> void visitActiveBBox(BBoxOp& op) const { mRoot.visitActiveBBox(op); }
 
     /*! Traverse this tree in depth-first order, and at each node call the given functor
      *  with a @c DenseIterator (see Iterator.h) that points to either a child node or a
@@ -1427,6 +1427,7 @@ Tree<RootNodeType>::pruneInactive()
     this->pruneInactive(this->background());
 }
 
+
 template<typename RootNodeType>
 inline void
 Tree<RootNodeType>::pruneLevelSet()
@@ -1435,6 +1436,24 @@ Tree<RootNodeType>::pruneLevelSet()
     this->pruneOp(op);
 }
 
+
+template<typename RootNodeType>
+inline void
+Tree<RootNodeType>::addTile(Index level, const Coord& xyz,
+                            const ValueType& value, bool active)
+{
+    mRoot.addTile(level, xyz, value, active);
+}
+
+template<typename RootNodeType>
+template<typename NodeT>
+inline NodeT*
+Tree<RootNodeType>::stealNode(const Coord& xyz, const ValueType& value, bool active)
+{
+    this->clearAllAccessors();
+    return mRoot.template stealNode<NodeT>(xyz, value, active);
+}
+
 template<typename RootNodeType>
 inline typename RootNodeType::LeafNodeType*
 Tree<RootNodeType>::touchLeaf(const Coord& xyz)
@@ -1442,6 +1461,7 @@ Tree<RootNodeType>::touchLeaf(const Coord& xyz)
     return mRoot.touchLeaf(xyz);
 }
 
+
 template<typename RootNodeType>
 inline typename RootNodeType::LeafNodeType*
 Tree<RootNodeType>::probeLeaf(const Coord& xyz)
@@ -1449,6 +1469,7 @@ Tree<RootNodeType>::probeLeaf(const Coord& xyz)
     return mRoot.probeLeaf(xyz);
 }
 
+
 template<typename RootNodeType>
 inline const typename RootNodeType::LeafNodeType*
 Tree<RootNodeType>::probeConstLeaf(const Coord& xyz) const
@@ -1456,6 +1477,7 @@ Tree<RootNodeType>::probeConstLeaf(const Coord& xyz) const
     return mRoot.probeConstLeaf(xyz);
 }
 
+
 ////////////////////////////////////////
 
 
@@ -1468,6 +1490,14 @@ Tree<RootNodeType>::fill(const CoordBBox& bbox, const ValueType& value, bool act
 }
 
 
+template<typename RootNodeType>
+inline void
+Tree<RootNodeType>::signedFloodFill(const ValueType& outside, const ValueType& inside)
+{
+    mRoot.signedFloodFill(outside, inside);
+}
+
+
 template<typename RootNodeType>
 Metadata::Ptr
 Tree<RootNodeType>::getBackgroundValue() const
@@ -1487,6 +1517,38 @@ Tree<RootNodeType>::getBackgroundValue() const
 ////////////////////////////////////////
 
 
+template<typename RootNodeType>
+inline void
+Tree<RootNodeType>::merge(Tree& other)
+{
+    this->clearAllAccessors();
+    other.clearAllAccessors();
+    mRoot.merge(other.mRoot);
+}
+
+
+template<typename RootNodeType>
+inline void
+Tree<RootNodeType>::voxelizeActiveTiles()
+{
+    this->clearAllAccessors();
+    mRoot.voxelizeActiveTiles();
+}
+
+
+template<typename RootNodeType>
+template<typename OtherRootNodeType>
+inline void
+Tree<RootNodeType>::topologyUnion(const Tree<OtherRootNodeType>& other)
+{
+    this->clearAllAccessors();
+    mRoot.topologyUnion(other.getRootNode());
+}
+
+
+////////////////////////////////////////
+
+
 /// @brief Helper class to adapt a three-argument (a, b, result) CombineOp functor
 /// into a single-argument functor that accepts a CombineArgs struct
 template<typename ValueT, typename CombineOp>
