@@ -33,73 +33,91 @@
 #ifndef OPENVDB_TREE_UTIL_HAS_BEEN_INCLUDED
 #define OPENVDB_TREE_UTIL_HAS_BEEN_INCLUDED
 
-#include <openvdb/math/Math.h>//for zeroVal
+#include <openvdb/math/Math.h> // for zeroVal
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 namespace tree {
 
-/// @brief Helper class for the tree nodes to replace constant tree
-/// branches (to within the provided tolerance) with a more memory
-/// efficient tile.   
-template<typename ValueType>
-struct TolerancePrune {
-    TolerancePrune(const ValueType &tol) : tolerance(tol) {}
-    template <typename ChildType>
-    bool operator()(ChildType &child) {
+/// @brief Helper class for use with Tree::pruneOp() to replace constant branches
+/// (to within the provided tolerance) with more memory-efficient tiles
+template<typename ValueType, Index TerminationLevel = 0>
+struct TolerancePrune
+{
+    TolerancePrune(const ValueType& tol): tolerance(tol) {}
+
+    template<typename ChildType>
+    bool operator()(ChildType& child)
+    {
+        return (ChildType::LEVEL < TerminationLevel) ? false : this->isConstant(child);
+    }
+
+    template<typename ChildType>
+    bool isConstant(ChildType& child)
+    {
         child.pruneOp(*this);
         return child.isConstant(value, state, tolerance);
     }
-    bool      state;
-    ValueType value;
+
+    bool            state;
+    ValueType       value;
     const ValueType tolerance;
 };
-    
-/// @brief Helper class for the tree nodes to replace inactive tree
-/// branches with a more memory efficient inactive tiles with the
-/// provided value. Specialized but faster then the tolerance based
-/// pruning defined above.
+
+
+/// @brief Helper class for use with Tree::pruneOp() to replace inactive branches
+/// with more memory-efficient inactive tiles with the provided value
+/// @details This is more specialized but faster than a TolerancePrune.
 template<typename ValueType>
-struct InactivePrune {
-    InactivePrune(const ValueType &val) : value(val) {}
+struct InactivePrune
+{
+    InactivePrune(const ValueType& val): value(val) {}
+
     template <typename ChildType>
-    bool operator()(ChildType &child) const {
+    bool operator()(ChildType& child) const
+    {
         child.pruneOp(*this);
         return child.isInactive();
     }
+
     static const bool state = false;
     const ValueType   value;
-};    
-    
-/// @brief Prune any descendants whose values are all inactive and replace them
-/// with inactive tiles having values with a magnitude equal to the background 
-/// value and a sign equal to the first value encountered in the (inactive) child.
+};
+
+
+/// @brief Helper class for use with Tree::pruneOp() to prune any branches
+/// whose values are all inactive and replace each with an inactive tile
+/// whose value is equal in magnitude to the background value and whose sign
+/// is equal to that of the first value encountered in the (inactive) child
 ///
-/// @note This method is faster then tolerance based prune and
-/// useful for narrow-band level set applications where inactive
-/// values are limited to either an inside or outside value. Also note
-/// that this methods
+/// @details This operation is faster than a TolerancePrune and useful for
+/// narrow-band level set applications where inactive values are limited
+/// to either the inside or the outside value.
 template<typename ValueType>
-struct LevelSetPrune {
-    LevelSetPrune(const ValueType &background) : outside(background) {}
+struct LevelSetPrune
+{
+    LevelSetPrune(const ValueType& background): outside(background) {}
+
     template <typename ChildType>
-    bool operator()(ChildType &child) {
+    bool operator()(ChildType& child)
+    {
         child.pruneOp(*this);
         if (!child.isInactive()) return false;
         value = child.getFirstValue() < zeroVal<ValueType>() ? -outside : outside;
         return true;
     }
+
     static const bool state = false;
     const ValueType   outside;
     ValueType         value;
 };
-    
+
 } // namespace tree
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-#endif // OPENVDB_TREE_TREE_HAS_BEEN_INCLUDED
+#endif // OPENVDB_TREE_UTIL_HAS_BEEN_INCLUDED
 
 // Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
