@@ -312,7 +312,24 @@ public:
     /// that node.  [Mainly for internal use]
     template<typename NodeType>
     void eraseNode() { LockT lock(mMutex); NodeType* node = NULL; mCache.erase(node); }
-
+    
+    /// @brief Add the specified leaf to this tree, possibly creating a child branch
+    /// in the process.  If the leaf node already exists, replace it.
+    void addLeaf(LeafNodeT* leaf)
+    {
+        LockT lock(mMutex);
+        mCache.addLeaf(leaf);
+    }
+    
+    /// @brief Add a tile at the specified tree level that contains
+    /// xyz, possibly creating a child branch in the process. If a
+    /// Node that contains xyz already exists it is replaced by a tile.
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool state)
+    {
+        LockT lock(mMutex);
+        mCache.addTile(level, xyz, value, state);
+    }
+    
     /// @brief @return the leaf node that contains voxel (x, y, z) and
     /// if it doesn't exist, create it, but preserve the values and
     /// active states of all voxels.
@@ -335,11 +352,15 @@ public:
 
     /// @brief @return a const pointer to the leaf node that contains
     /// voxel (x, y, z) and if it doesn't exist, return NULL.
-    const LeafNodeT* probeConstLeaf(const Coord& xyz)
+    const LeafNodeT* probeConstLeaf(const Coord& xyz) const
     {
         LockT lock(mMutex);
         return mCache.probeConstLeaf(xyz);
     }
+    const LeafNodeT* probeLeaf(const Coord& xyz) const
+    {
+        return this->probeConstLeaf(xyz);
+    }
 
     /// Remove all nodes from this cache, then reinsert the root node.
     virtual void clear()
@@ -563,7 +584,29 @@ public:
         }
         return mNext.getValue(xyz);
     }
-
+    
+    void addLeaf(LeafNodeType* leaf)
+    {
+        BOOST_STATIC_ASSERT(!TreeCacheT::IsConstTree);
+        if (NodeType::LEVEL == 0) return;
+        if (this->isHashed(leaf->origin())) {
+            assert(mNode);
+            return const_cast<NodeType*>(mNode)->addLeafAndCache(leaf, *mParent);
+        }
+        mNext.addLeaf(leaf);
+    }
+    
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool state)
+    {
+        BOOST_STATIC_ASSERT(!TreeCacheT::IsConstTree);
+        if (NodeType::LEVEL < level) return;
+        if (this->isHashed(xyz)) {
+            assert(mNode);
+            return const_cast<NodeType*>(mNode)->addTileAndCache(level, xyz, value, state, *mParent);
+        }
+        mNext.addTile(level, xyz, value, state);
+    }
+    
     LeafNodeType* touchLeaf(const Coord& xyz)
     {
         BOOST_STATIC_ASSERT(!TreeCacheT::IsConstTree);
@@ -750,7 +793,21 @@ public:
         node = const_cast<RootNodeType*>(mRoot);
     }
     void getNode(const RootNodeType*& node) const { node = mRoot; }
-
+    
+    void addLeaf(LeafNodeType* leaf)
+    {
+        assert(mRoot);
+        BOOST_STATIC_ASSERT(!TreeCacheT::IsConstTree);
+        const_cast<RootNodeType*>(mRoot)->addLeafAndCache(leaf, *mParent);
+    }
+    
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool state)
+    {
+        assert(mRoot);
+        BOOST_STATIC_ASSERT(!TreeCacheT::IsConstTree);
+        const_cast<RootNodeType*>(mRoot)->addTileAndCache(level, xyz, value, state, *mParent);
+    }
+    
     LeafNodeType* touchLeaf(const Coord& xyz)
     {
         assert(mRoot);
@@ -973,6 +1030,25 @@ public:
     /// the node containing voxel (x, y, z).  [Mainly for internal use]
     template<typename NodeT> void insertNode(const Coord&, NodeT&) {}
 
+    /// @brief Add the specified leaf to this tree, possibly creating a child branch
+    /// in the process.  If the leaf node already exists, replace it.
+    void addLeaf(LeafNodeT* leaf)
+    {
+        assert(BaseT::mTree);
+        BOOST_STATIC_ASSERT(!BaseT::IsConstTree);
+        BaseT::mTree->root().addLeaf(leaf);
+    }
+    
+    /// @brief Add a tile at the specified tree level that contains
+    /// xyz, possibly creating a child branch in the process. If a
+    /// Node that contains xyz already exists it is replaced by a tile.
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool state)
+    {
+        assert(BaseT::mTree);
+        BOOST_STATIC_ASSERT(!BaseT::IsConstTree);
+        BaseT::mTree->root().addTile(level, xyz, value, state);
+    }
+
     /// If a node of the given type exists in the cache, remove it, so that
     /// isCached(xyz) returns @c false for any voxel (x, y, z) contained in
     /// that node.  [Mainly for internal use]
@@ -992,12 +1068,17 @@ public:
         return BaseT::mTree->probeLeaf(xyz);
     }
 
-    const LeafNodeT* probeConstLeaf(const Coord& xyz)
+    const LeafNodeT* probeConstLeaf(const Coord& xyz) const
     {
         assert(BaseT::mTree);
         return BaseT::mTree->probeConstLeaf(xyz);
     }
 
+    const LeafNodeT* probeLeaf(const Coord& xyz) const
+    {
+        return this->probeConstLeaf(xyz);
+    }
+
     /// Remove all nodes from this cache, then reinsert the root node.
     virtual void clear() {}
 
@@ -1219,6 +1300,25 @@ public:
         const NodeT* node = NULL;
         this->eraseNode(node);
     }
+    
+    /// @brief Add the specified leaf to this tree, possibly creating a child branch
+    /// in the process.  If the leaf node already exists, replace it.
+    void addLeaf(LeafNodeT* leaf)
+    {
+        assert(BaseT::mTree);
+        BOOST_STATIC_ASSERT(!BaseT::IsConstTree);
+        BaseT::mTree->root().addLeaf(leaf);
+    }
+    
+    /// @brief Add a tile at the specified tree level that contains
+    /// xyz, possibly creating a child branch in the process. If a
+    /// Node that contains xyz already exists it is replaced by a tile.
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool state)
+    {
+        assert(BaseT::mTree);
+        BOOST_STATIC_ASSERT(!BaseT::IsConstTree);
+        BaseT::mTree->root().addTile(level, xyz, value, state);
+    }
 
     /// @brief @return the leaf node that contains voxel (x, y, z) and
     /// if it doesn't exist, create it, but preserve the values and
@@ -1252,15 +1352,16 @@ public:
 
     /// @brief @return a const pointer to the leaf node that contains
     /// voxel (x, y, z) and if it doesn't exist, return NULL.
-    const LeafNodeT* probeConstLeaf(const Coord& xyz)
+    const LeafNodeT* probeConstLeaf(const Coord& xyz) const
     {
         assert(BaseT::mTree);
         if (this->isHashed(xyz)) {
             assert(mNode0);
-            return mNode0->probeLeafAndCache(xyz, *this);
+            return mNode0->probeConstLeafAndCache(xyz, this->self());
         }
-        return BaseT::mTree->getRootNode().probeConstLeafAndCache(xyz, *this);
+        return BaseT::mTree->getRootNode().probeConstLeafAndCache(xyz, this->self());
     }
+    const LeafNodeT* probeLeaf(const Coord& xyz) const { return this->probeConstLeaf(xyz); }
 
     /// Remove all the cached nodes and invalidate the corresponding hash-keys.
     virtual void clear()
@@ -1568,7 +1669,34 @@ public:
         const NodeT* node = NULL;
         this->eraseNode(node);
     }
-
+    
+    /// @brief Add the specified leaf to this tree, possibly creating a child branch
+    /// in the process.  If the leaf node already exists, replace it.
+    void addLeaf(LeafNodeT* leaf)
+    {
+        assert(BaseT::mTree);
+        BOOST_STATIC_ASSERT(!BaseT::IsConstTree);
+        if (this->isHashed1(leaf->origin())) {
+            assert(mNode1);
+            return const_cast<NodeT1*>(mNode1)->addLeafAndCache(leaf, *this);
+        }
+        BaseT::mTree->root().addLeafAndCache(leaf, *this);
+    }
+    
+    /// @brief Add a tile at the specified tree level that contains
+    /// xyz, possibly creating a child branch in the process. If a
+    /// Node that contains xyz already exists it is replaced by a tile.
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool state)
+    {
+        assert(BaseT::mTree);
+        BOOST_STATIC_ASSERT(!BaseT::IsConstTree);
+        if (this->isHashed1(xyz)) {
+            assert(mNode1);
+            return const_cast<NodeT1*>(mNode1)->addTileAndCache(level, xyz, value, state, *this);
+        }
+        BaseT::mTree->root().addTileAndCache(level, xyz, value, state, *this);
+    }
+    
     /// @brief @return the leaf node that contains voxel (x, y, z) and
     /// if it doesn't exist, create it, but preserve the values and
     /// active states of all voxels.
@@ -1607,18 +1735,19 @@ public:
 
     /// @brief @return a const pointer to the leaf node that contains
     /// voxel (x, y, z) and if it doesn't exist, return NULL.
-    const LeafNodeT* probeConstLeaf(const Coord& xyz)
+    const LeafNodeT* probeConstLeaf(const Coord& xyz) const
     {
         assert(BaseT::mTree);
         if (this->isHashed0(xyz)) {
             assert(mNode0);
-            return mNode0->probeConstLeafAndCache(xyz, *this);
+            return mNode0->probeConstLeafAndCache(xyz, this->self());
         } else if (this->isHashed1(xyz)) {
             assert(mNode1);
-            return mNode1->probeConstLeafAndCache(xyz, *this);
+            return mNode1->probeConstLeafAndCache(xyz, this->self());
         }
-        return BaseT::mTree->getRootNode().probeConstLeafAndCache(xyz, *this);
+        return BaseT::mTree->getRootNode().probeConstLeafAndCache(xyz, this->self());
     }
+    const LeafNodeT* probeLeaf(const Coord& xyz) const { return this->probeConstLeaf(xyz); }
 
     /// Remove all the cached nodes and invalidate the corresponding hash-keys.
     virtual void clear()
@@ -1983,6 +2112,39 @@ public:
         this->eraseNode(node);
     }
 
+    /// @brief Add the specified leaf to this tree, possibly creating a child branch
+    /// in the process.  If the leaf node already exists, replace it.
+    void addLeaf(LeafNodeT* leaf)
+    {
+        assert(BaseT::mTree);
+        BOOST_STATIC_ASSERT(!BaseT::IsConstTree);
+        if (this->isHashed1(leaf->origin())) {
+            assert(mNode1);
+            return const_cast<NodeT1*>(mNode1)->addLeafAndCache(leaf, *this);
+        } else if (this->isHashed2(leaf->origin())) {
+            assert(mNode2);
+            return const_cast<NodeT2*>(mNode2)->addLeafAndCache(leaf, *this);
+        }
+        BaseT::mTree->root().addLeafAndCache(leaf, *this);
+    }
+    
+    /// @brief Add a tile at the specified tree level that contains
+    /// xyz, possibly creating a child branch in the process. If a
+    /// Node that contains xyz already exists it is replaced by a tile.
+    void addTile(Index level, const Coord& xyz, const ValueType& value, bool state)
+    {
+        assert(BaseT::mTree);
+        BOOST_STATIC_ASSERT(!BaseT::IsConstTree);
+        if (this->isHashed1(xyz)) {
+            assert(mNode1);
+            return const_cast<NodeT1*>(mNode1)->addTileAndCache(level, xyz, value, state, *this);
+        } if (this->isHashed2(xyz)) {
+            assert(mNode2);
+            return const_cast<NodeT2*>(mNode2)->addTileAndCache(level, xyz, value, state, *this);
+        }
+        BaseT::mTree->root().addTileAndCache(level, xyz, value, state, *this);
+    }
+
     /// @brief @return the leaf node that contains voxel (x, y, z) and
     /// if it doesn't exist, create it, but preserve the values and
     /// active states of all voxels.
@@ -2027,21 +2189,22 @@ public:
 
     /// @brief @return a const pointer to the leaf node that contains
     /// voxel (x, y, z) and if it doesn't exist, return NULL.
-    const LeafNodeT* probeConstLeaf(const Coord& xyz)
+    const LeafNodeT* probeConstLeaf(const Coord& xyz) const
     {
         assert(BaseT::mTree);
         if (this->isHashed0(xyz)) {
             assert(mNode0);
-            return mNode0->probeConstLeafAndCache(xyz, *this);
+            return mNode0->probeConstLeafAndCache(xyz, this->self());
         } else if (this->isHashed1(xyz)) {
             assert(mNode1);
-            return mNode1->probeConstLeafAndCache(xyz, *this);
+            return mNode1->probeConstLeafAndCache(xyz, this->self());
         } else if (this->isHashed2(xyz)) {
             assert(mNode2);
-            return mNode2->probeConstLeafAndCache(xyz, *this);
+            return mNode2->probeConstLeafAndCache(xyz, this->self());
         }
-        return BaseT::mTree->getRootNode().probeConstLeafAndCache(xyz, *this);
+        return BaseT::mTree->getRootNode().probeConstLeafAndCache(xyz, this->self());
     }
+    const LeafNodeT* probeLeaf(const Coord& xyz) const { return this->probeConstLeaf(xyz); }
 
     /// Remove all the cached nodes and invalidate the corresponding hash-keys.
     virtual void clear()
