@@ -143,7 +143,7 @@ TestCurl::testISCurl()
     --dim;//ignore boundary curl vectors
     // test unit space operators
     Accessor inConstAccessor = inGrid->getConstAccessor();
-     for (int x = -dim; x<dim; ++x) {
+    for (int x = -dim; x<dim; ++x) {
         for (int y = -dim; y<dim; ++y) {
             for (int z = -dim; z<dim; ++z) {
                 Coord xyz(x,y,z);
@@ -170,7 +170,7 @@ TestCurl::testISCurl()
         }
     }
 
-     --dim;//ignore boundary curl vectors
+    --dim;//ignore boundary curl vectors
     for (int x = -dim; x<dim; ++x) {
         for (int y = -dim; y<dim; ++y) {
             for (int z = -dim; z<dim; ++z) {
@@ -206,25 +206,24 @@ TestCurl::testISCurl()
                 Coord xyz(x,y,z);
                 VectorTree::ValueType v = inAccessor.getValue(xyz);
 
-                ASSERT_DOUBLES_EXACTLY_EQUAL( y,v[0]);
-                ASSERT_DOUBLES_EXACTLY_EQUAL(-x,v[1]);
-                ASSERT_DOUBLES_EXACTLY_EQUAL( 0,v[2]);
+                ASSERT_DOUBLES_EXACTLY_EQUAL( y, v[0]);
+                ASSERT_DOUBLES_EXACTLY_EQUAL(-x, v[1]);
+                ASSERT_DOUBLES_EXACTLY_EQUAL( 0, v[2]);
                 v = math::ISCurl<math::CD_6TH>::result(inConstAccessor, xyz);
 
-                ASSERT_DOUBLES_EXACTLY_EQUAL( 0,v[0]);
-                ASSERT_DOUBLES_EXACTLY_EQUAL( 0,v[1]);
-                ASSERT_DOUBLES_EXACTLY_EQUAL(-2,v[2]);
-
+                ASSERT_DOUBLES_EXACTLY_EQUAL( 0, v[0]);
+                ASSERT_DOUBLES_EXACTLY_EQUAL( 0, v[1]);
+                ASSERT_DOUBLES_EXACTLY_EQUAL(-2, v[2]);
 
                 v = math::ISCurl<math::FD_3RD>::result(inConstAccessor, xyz);
-                ASSERT_DOUBLES_EXACTLY_EQUAL( 0,v[0]);
-                ASSERT_DOUBLES_EXACTLY_EQUAL( 0,v[1]);
-                CPPUNIT_ASSERT_DOUBLES_EQUAL(-2,v[2], /*tolerance=*/0.00001);
+                ASSERT_DOUBLES_EXACTLY_EQUAL( 0, v[0]);
+                CPPUNIT_ASSERT_DOUBLES_EQUAL( 0, v[1], /*tolerance=*/0.00001);
+                CPPUNIT_ASSERT_DOUBLES_EQUAL(-2, v[2], /*tolerance=*/0.00001);
 
                 v = math::ISCurl<math::BD_3RD>::result(inConstAccessor, xyz);
-                ASSERT_DOUBLES_EXACTLY_EQUAL( 0,v[0]);
-                ASSERT_DOUBLES_EXACTLY_EQUAL( 0,v[1]);
-                CPPUNIT_ASSERT_DOUBLES_EQUAL(-2,v[2], /*tolerance=*/0.00001);
+                ASSERT_DOUBLES_EXACTLY_EQUAL( 0, v[0]);
+                ASSERT_DOUBLES_EXACTLY_EQUAL( 0, v[1]);
+                CPPUNIT_ASSERT_DOUBLES_EQUAL(-2, v[2], /*tolerance=*/0.00001);
             }
         }
     }
