@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -215,11 +215,12 @@ TestLaplacian::testWSLaplacian()
 
 }
 
+
 void
 TestLaplacian::testWSLaplacianFrustum()
 {
     using namespace openvdb;
-    
+
     // Create a Frustum Map:
 
     openvdb::BBoxd bbox(Vec3d(0), Vec3d(100));
@@ -230,18 +231,18 @@ TestLaplacian::testWSLaplacianFrustum()
     math::Vec3d trans(2,2,2);
     math::NonlinearFrustumMap::Ptr map =
         boost::static_pointer_cast<math::NonlinearFrustumMap, math::MapBase>(
-            frustum.preScale(Vec3d(10,10,10))->translate(trans));
+            frustum.preScale(Vec3d(10,10,10))->postTranslate(trans));
 
     CPPUNIT_ASSERT(!map->hasUniformScale());
 
     math::Vec3d result;
     result = map->voxelSize();
-    
+
     CPPUNIT_ASSERT( math::isApproxEqual(result.x(), 0.1));
     CPPUNIT_ASSERT( math::isApproxEqual(result.y(), 0.1));
     CPPUNIT_ASSERT( math::isApproxEqual(result.z(), 0.5, 0.0001));
 
-    
+
     // Create a tree
     FloatGrid::Ptr grid = FloatGrid::create(/*background=*/0.0);
     FloatTree& tree = grid->tree();
@@ -261,18 +262,16 @@ TestLaplacian::testWSLaplacianFrustum()
     }
 
     const Coord testloc(16,16,16);
-    float test_result  
+    float test_result
         = math::Laplacian<math::NonlinearFrustumMap, math::CD_SECOND>::result(*map, tree, testloc);
     float expected_result =  -3.f * tree.getValue(testloc);
-    
+
     // The exact solution of Laplacian( cos(x)sin(y)cos(z) ) = -3 cos(x) sin(y) cos(z)
 
     CPPUNIT_ASSERT( math::isApproxEqual(test_result, expected_result, /*tolerance=*/0.02f) );
 
 }
-    
-            
-    
+
 
 void
 TestLaplacian::testWSLaplacianStencil()
@@ -436,6 +435,6 @@ TestLaplacian::testLaplacianTool()
         2.0/20.0, lap->getConstAccessor().getValue(xyz),0.01);// 2/distance from center
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
