@@ -47,6 +47,7 @@ public:
     CPPUNIT_TEST(testPrimitiveVoxelRatio);
     CPPUNIT_TEST(testIntersectingVoxelCleaner);
     CPPUNIT_TEST(testShellVoxelCleaner);
+    CPPUNIT_TEST(testConversion);
     CPPUNIT_TEST_SUITE_END();
 
     void testUtils();
@@ -54,6 +55,7 @@ public:
     void testPrimitiveVoxelRatio();
     void testIntersectingVoxelCleaner();
     void testShellVoxelCleaner();
+    void testConversion();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestMeshToVolume);
@@ -109,7 +111,7 @@ TestMeshToVolume::testVoxelizer()
 {
     std::vector<openvdb::Vec3s> pointList;
     std::vector<openvdb::Vec4I> polygonList;
-    
+
     typedef openvdb::tools::internal::MeshVoxelizer<openvdb::FloatTree> MeshVoxelizer;
 
     // CASE 1: One triangle
@@ -123,7 +125,7 @@ TestMeshToVolume::testVoxelizer()
     {
         MeshVoxelizer voxelizer(pointList, polygonList);
         voxelizer.runParallel();
-        
+
         // Check for mesh intersecting voxels
         CPPUNIT_ASSERT(13== voxelizer.intersectionTree().activeVoxelCount());
 
@@ -189,6 +191,7 @@ TestMeshToVolume::testVoxelizer()
     }
 }
 
+
 void
 TestMeshToVolume::testPrimitiveVoxelRatio()
 {
@@ -201,7 +204,7 @@ TestMeshToVolume::testPrimitiveVoxelRatio()
     pointList.push_back(openvdb::Vec3s(0.0, 100.0, 0.0));
 
     polygonList.push_back(openvdb::Vec4I(0, 1, 2, openvdb::util::INVALID_IDX));
-    
+
     openvdb::tools::internal::MeshVoxelizer<openvdb::FloatTree>
         voxelizer(pointList, polygonList);
 
@@ -210,19 +213,20 @@ TestMeshToVolume::testPrimitiveVoxelRatio()
     CPPUNIT_ASSERT(0 != voxelizer.intersectionTree().activeVoxelCount());
 }
 
+
 void
 TestMeshToVolume::testIntersectingVoxelCleaner()
 {
     // Empty tree's
-    
+
     openvdb::FloatTree distTree(std::numeric_limits<float>::max());
     openvdb::BoolTree intersectionTree(false);
     openvdb::Int32Tree indexTree(openvdb::util::INVALID_IDX);
-    
+
     openvdb::tree::ValueAccessor<openvdb::FloatTree> distAcc(distTree);
     openvdb::tree::ValueAccessor<openvdb::BoolTree> intersectionAcc(intersectionTree);
     openvdb::tree::ValueAccessor<openvdb::Int32Tree> indexAcc(indexTree);
-    
+
     // Add a row of intersecting voxels surrounded by both positive and negative distance values.
     for (int i = 0; i < 10; ++i) {
         for (int j = -1; j < 2; ++j) {
@@ -236,9 +240,9 @@ TestMeshToVolume::testIntersectingVoxelCleaner()
     unsigned int numIVoxels = intersectionTree.activeVoxelCount();
     unsigned int numCPVoxels = indexTree.activeVoxelCount();
 
-    {    
+    {
         openvdb::tree::LeafManager<openvdb::BoolTree> leafs(intersectionTree);
-        
+
         openvdb::tools::internal::IntersectingVoxelCleaner<openvdb::FloatTree>
             cleaner(distTree, indexTree, intersectionTree, leafs);
 
@@ -263,7 +267,7 @@ TestMeshToVolume::testIntersectingVoxelCleaner()
 
     {
         openvdb::tree::LeafManager<openvdb::BoolTree> leafs(intersectionTree);
-        
+
         openvdb::tools::internal::IntersectingVoxelCleaner<openvdb::FloatTree>
             cleaner(distTree, indexTree, intersectionTree, leafs);
 
@@ -275,15 +279,16 @@ TestMeshToVolume::testIntersectingVoxelCleaner()
     CPPUNIT_ASSERT(numCPVoxels == indexTree.activeVoxelCount());
 }
 
+
 void
 TestMeshToVolume::testShellVoxelCleaner()
 {
     // Empty tree's
-    
+
     openvdb::FloatTree distTree(std::numeric_limits<float>::max());
     openvdb::BoolTree intersectionTree(false);
     openvdb::Int32Tree indexTree(openvdb::util::INVALID_IDX);
-    
+
     openvdb::tree::ValueAccessor<openvdb::FloatTree> distAcc(distTree);
     openvdb::tree::ValueAccessor<openvdb::BoolTree> intersectionAcc(intersectionTree);
     openvdb::tree::ValueAccessor<openvdb::Int32Tree> indexAcc(indexTree);
@@ -330,6 +335,44 @@ TestMeshToVolume::testShellVoxelCleaner()
     CPPUNIT_ASSERT(0 == indexTree.activeVoxelCount());;
 }
 
+
+void
+TestMeshToVolume::testConversion()
+{
+    using namespace openvdb;
+
+    std::vector<Vec3s> points;
+    std::vector<Vec4I> quads;
+
+    // cube vertices
+    points.push_back(Vec3s(2, 2, 2)); // 0       6--------7
+    points.push_back(Vec3s(5, 2, 2)); // 1      /|       /|
+    points.push_back(Vec3s(2, 5, 2)); // 2     2--------3 |
+    points.push_back(Vec3s(5, 5, 2)); // 3     | |      | |
+    points.push_back(Vec3s(2, 2, 5)); // 4     | 4----- |-5
+    points.push_back(Vec3s(5, 2, 5)); // 5     |/       |/
+    points.push_back(Vec3s(2, 5, 5)); // 6     0--------1
+    points.push_back(Vec3s(5, 5, 5)); // 7
+
+    // cube faces
+    quads.push_back(Vec4I(0, 1, 3, 2)); // front
+    quads.push_back(Vec4I(5, 4, 6, 7)); // back
+    quads.push_back(Vec4I(0, 2, 6, 4)); // left
+    quads.push_back(Vec4I(1, 5, 7, 3)); // right
+    quads.push_back(Vec4I(2, 3, 7, 6)); // top
+    quads.push_back(Vec4I(0, 4, 5, 1)); // bottom
+
+    FloatGrid::Ptr grid = tools::meshToLevelSet<FloatGrid>(
+        *math::Transform::createLinearTransform(), points, quads);
+
+    //io::File("/tmp/cube.vdb").write(GridPtrVec(1, grid));
+
+    CPPUNIT_ASSERT(grid.get() != NULL);
+    CPPUNIT_ASSERT_EQUAL(int(GRID_LEVEL_SET), int(grid->getGridClass()));
+    CPPUNIT_ASSERT_EQUAL(1, int(grid->baseTree().leafCount()));
+    /// @todo validate output
+}
+
 // Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
