@@ -43,9 +43,11 @@ class TestVolumeToMesh: public CppUnit::TestCase
 public:
     CPPUNIT_TEST_SUITE(TestVolumeToMesh);
     CPPUNIT_TEST(testAuxData);
+    CPPUNIT_TEST(testConversion);
     CPPUNIT_TEST_SUITE_END();
 
     void testAuxData();
+    void testConversion();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestVolumeToMesh);
@@ -98,6 +100,35 @@ TestVolumeToMesh::testAuxData()
     CPPUNIT_ASSERT(!bool(flags & openvdb::tools::internal::ZEDGE));
 }
 
+
+void
+TestVolumeToMesh::testConversion()
+{
+    using namespace openvdb;
+
+    FloatGrid::Ptr grid = createGrid<FloatGrid>(/*background=*/1);
+    grid->fill(CoordBBox(Coord(0), Coord(7)), 0.0);
+    grid->fill(CoordBBox(Coord(1), Coord(6)), -1.0);
+
+    std::vector<Vec3s> points;
+    std::vector<Vec4I> quads;
+    std::vector<Vec3I> triangles;
+
+    openvdb::tools::volumeToMesh(*grid, points, quads);
+    CPPUNIT_ASSERT(points.size() >= 4);
+    CPPUNIT_ASSERT(!quads.empty());
+    /// @todo validate output
+
+    points.clear();
+    quads.clear();
+    triangles.clear();
+
+    tools::volumeToMesh(*grid, points, triangles, quads, /*isovalue=*/0.01, /*adaptivity=*/0);
+    CPPUNIT_ASSERT(points.size() >= 3);
+    CPPUNIT_ASSERT(!triangles.empty() || !quads.empty());
+    /// @todo validate output
+}
+
 // Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
