@@ -73,11 +73,14 @@ class LeafTopologyIntOp
 public:
     LeafTopologyIntOp(const TreeType2& tree): mOtherTree(&tree) {}
 
-    inline void operator()(const typename TreeType1::LeafIter& lIter) const {
-        tree::ValueAccessor<const TreeType2> otherValueAccessor(*mOtherTree);
-        typename TreeType1::LeafNodeType::ValueOnIter vIter = lIter.getLeaf()->beginValueOn();
-        for ( ; vIter; ++vIter) {
-            if (!otherValueAccessor.isValueOn(vIter.getCoord())) vIter.setValueOff();
+    inline void operator()(const typename TreeType1::LeafIter& lIter) const
+    {
+        const Coord xyz = lIter->getOrigin();
+        const typename TreeType2::LeafNodeType* leaf = mOtherTree->probeConstLeaf(xyz);
+        if (leaf) {//leaf node
+            lIter->topologyIntersection(*leaf, zeroVal<typename TreeType1::ValueType>());
+        } else if (!mOtherTree->isValueOn(xyz)) {//inactive tile
+            lIter->setValuesOff();
         }
     }
 
@@ -94,11 +97,14 @@ class LeafTopologyDiffOp
 public:
     LeafTopologyDiffOp(const TreeType2& tree): mOtherTree(&tree) {}
 
-    inline void operator()(const typename TreeType1::LeafIter& lIter) const {
-        tree::ValueAccessor<const TreeType2> otherValueAccessor(*mOtherTree);
-        typename TreeType1::LeafNodeType::ValueOnIter vIter = lIter.getLeaf()->beginValueOn();
-        for ( ; vIter; ++vIter) {
-            if (otherValueAccessor.isValueOn(vIter.getCoord())) vIter.setValueOff();
+    inline void operator()(const typename TreeType1::LeafIter& lIter) const
+    {
+        const Coord xyz = lIter->getOrigin();
+        const typename TreeType2::LeafNodeType* leaf = mOtherTree->probeConstLeaf(xyz);
+        if (leaf) {//leaf node
+            lIter->topologyDifference(*leaf, zeroVal<typename TreeType1::ValueType>());
+        } else if (mOtherTree->isValueOn(xyz)) {//active tile
+            lIter->setValuesOff();
         }
     }
 
